@@ -2,15 +2,20 @@
 #include <libmc.h>
 #ifdef _WIN32
 #include <windows.h>
+#else
+#include <unistd.h>
 #endif
 #define WAIT_TIME 5
 int main(int argc, char *argv[]) 
 {
   MCAgency_t agency;
   MCAgencyOptions_t options;
+  int port=5050;
+  int remote_port=5051;
+
   MC_InitializeAgencyOptions(&options);
-  MC_SetThreadOff(&options, MC_THREAD_CP);
-  agency = MC_Initialize(5050, &options);
+  MC_SetThreadOff(&options, MC_THREAD_CP); /* Turn off command prompt */
+  agency = MC_Initialize(port, &options);
 
   printf("MobileC Started\n");
   printf("Sending sleep agent...\n");
@@ -20,7 +25,7 @@ int main(int argc, char *argv[])
       agency,
       "sleep.xml",
       "localhost",
-      5051);
+      remote_port);
   printf("Sleeping for %d seconds.\n", WAIT_TIME);
 #ifndef _WIN32
   sleep(WAIT_TIME);
@@ -34,7 +39,7 @@ int main(int argc, char *argv[])
       agency,
       "wake.xml",
       "localhost",
-      5051);
+      remote_port);
   printf("Terminating...\n");
   MC_End(agency);
   return 0;
