@@ -8,10 +8,12 @@ int main(int argc, char *argv[])
 {
     MCAgency_t agency;
     MCAgencyOptions_t options;
+    int port=5050;
+    int remote_port=5051;
 
     MC_InitializeAgencyOptions(&options);
-    MC_SetThreadOff(&options, MC_THREAD_CP);
-    agency = MC_Initialize(5050, &options);
+    MC_SetThreadOff(&options, MC_THREAD_CP); /* Turn off command prompt */
+    agency = MC_Initialize(port, &options);
 
     printf("MobileC Started\n");
     printf("Sending sleep agent...\n");
@@ -21,7 +23,7 @@ int main(int argc, char *argv[])
         agency,
         "sleep.xml",
         "localhost",
-        5051);
+        remote_port);
     printf("Sleeping for %d seconds.\n", WAIT_TIME);
 #ifndef _WIN32
     sleep(WAIT_TIME);
@@ -35,7 +37,7 @@ int main(int argc, char *argv[])
         agency,
         "wake.xml",
         "localhost",
-        5051);
+        remote_port);
     MC_End(agency);
     return 0;
 }
