@@ -13,15 +13,18 @@ int main(int argc, char *argv[])
     MCAgency_t agency;
     MCAgencyOptions_t options;
     int i;
+    int port=5051;
     MC_InitializeAgencyOptions(&options);
-    /* We want _all_ the threads on: EXCEPT, the command prompt thread */
+
     for (i = 0; i < MC_THREAD_ALL; i++) {
         MC_SetThreadOn(&options, i);
     }
+    /* If the following line is uncommented, the command prompt
+     * will be disabled. */
     /*MC_SetThreadOff(&options, MC_THREAD_CP);  */
 
     agency = MC_Initialize(
-            5051,
+            port,
             &options);
 
     /* Wait forever... */
