@@ -4,16 +4,18 @@ int main(int argc, char *argv[])
 {
     MCAgency_t agency;
     MCAgencyOptions_t options;
+    int port;
     int i;
     MC_InitializeAgencyOptions(&options);
-    /* We want _all_ the threads on: EXCEPT, the command prompt thread */
     for (i = 0; i < MC_THREAD_ALL; i++) {
         MC_SetThreadOn(&options, i);
     }
+    /* If the following line is uncommented, the command prompt will be
+     * turned off. */
  /*   MC_SetThreadOff(&options, MC_THREAD_CP);  */
 
     agency = MC_Initialize(
-            5052,
+            port,
             &options);
 
 
