@@ -9,6 +9,10 @@ using System.Windows.Forms;
 using System.Runtime.InteropServices;
 using LibMC;
 
+/** \file /LibMCGui/Form1.cs
+ * Program for testing miscellaneous functions in LibMC.NET.
+ */
+
 /**
  * \brief      Namespace for GUI-based Mobile-C example.
  * 
@@ -87,7 +91,16 @@ namespace LibMCTest
             if (ofd.ShowDialog() == DialogResult.OK)
             {
                 String filename = ofd.FileName;
-                Agency.SendAgentMigrationMessageFile(filename, "localhost", Agency.Port);
+                try
+                {
+                    Agency.SendAgentMigrationMessageFile(filename, 
+                        "localhost", Agency.Port);
+                }
+                catch (Exception ex)
+                {
+                    richTextBox1.Text += "Error sending file: " + ex.Message + "\n";
+                    richTextBox1.Text += "Is the local agency running?\n";
+                }
             }
         }
 
@@ -109,7 +122,8 @@ namespace LibMCTest
                 String filename = ofd.FileName;
                 try
                 {
-                    Agency.SendAgentMigrationMessageFile(filename, textBox1.Text, (int)numericUpDown2.Value);
+                    Agency.SendAgentMigrationMessageFile(filename, 
+                        textBox1.Text, (int)numericUpDown2.Value);
                 }
                 catch (Exception ex)
                 {
@@ -150,7 +164,8 @@ namespace LibMCTest
             {
                 temp = Agency.HaltAgency();
                 if (temp != 0)
-                    richTextBox1.Text += "Error halting agency: " + temp.ToString() + "\n";
+                    richTextBox1.Text += "Error halting agency: " + 
+                        temp.ToString() + "\n";
                 else
                     button3.Text = "Resume Agency";
             }
@@ -158,13 +173,15 @@ namespace LibMCTest
             {
                 temp = Agency.ResumeAgency();
                 if (temp != 0)
-                    richTextBox1.Text += "Error resuming agency: " + temp.ToString() + "\n";
+                    richTextBox1.Text += "Error resuming agency: " + 
+                        temp.ToString() + "\n";
                 else
                     button3.Text = "Halt Agency";
             }
             else
             {
-                richTextBox1.Text += "Error: Agency is in an unknown state: " + Agency.State.ToString();
+                richTextBox1.Text += "Error: Agency is in an unknown state: " + 
+                    Agency.State.ToString();
             }
             label3.Text = Agency.State.ToString();
         }
