@@ -35,13 +35,95 @@
 
 #include <embedch.h>
 
+#include "include/libmc.h"
 #include "include/agent.h"
 #include "include/mc_platform.h"
 #include "include/message.h"
 #include "include/agent_lib.h"
-#include "include/agent_return_data.h"
+#include "include/interpreter_variable_data.h"
 #include "include/xml_parser.h"
 
+int agent_AddPersistentVariable(agent_p agent, int task_num, const char* var_name)
+{
+  int i;
+  int size;
+  int data_type_size;
+  int progress;
+  interpreter_variable_data_t *agent_variable_data;
+  agent_variable_data = (interpreter_variable_data_t*)malloc(sizeof(interpreter_variable_data_t));
+  agent_variable_data->name = strdup(var_name);
+
+  /* Get the array data type */
+  agent_variable_data->data_type = Ch_DataType(
+      agent->agent_interp,
+      var_name );
+  /* Make sure the variable exists */
+  if (agent_variable_data->data_type == CH_UNDEFINETYPE) {
+    free(agent_variable_data);
+    fprintf(stderr, "Warning: agent %s attempted saving of invalid variable, %s. %s:%d\n",
+        agent->name, var_name, __FILE__, __LINE__);
+    return MC_ERR;
+  }
+  /* Get the array dimension */
+  agent_variable_data->array_dim = Ch_ArrayDim(
+      agent->agent_interp,
+      var_name );
+  /* Get the array extents */
+  agent_variable_data->array_extent = (int*)malloc(
+      sizeof(int) * agent_variable_data->array_dim );
+  for (i=0; i<agent_variable_data->array_dim; i++) {
+    agent_variable_data->array_extent[i] = 
+      Ch_ArrayExtent(
+          agent->agent_interp,
+          var_name,
+          i );
+  }
+  /* Finally, allocate and point returnData to the right place. */
+  size = 1;
+  for (i=0; i < agent_variable_data->array_dim; i++) {
+    size *= agent_variable_data->array_extent[i];
+  }
+
+  /* Now get the data type size */
+  CH_DATATYPE_SIZE(agent_variable_data->data_type, data_type_size);
+
+  agent_variable_data->data = (void*)malloc(size * data_type_size);
+  CHECK_NULL(agent_variable_data->data, exit(0));
+  /* Copy the data over from the agent */
+  /* For now, only support statically allocated global vars. */
+  progress = agent->datastate->task_progress;
+  i = 0;
+
+  if (agent_variable_data->array_dim == 0) {
+    memcpy(
+        agent_variable_data->data,
+        (void*)Ch_GlobalSymbolAddrByName(
+          agent->agent_interp,
+          var_name),
+        size*data_type_size
+        );
+
+  } else {
+    memcpy(
+        agent_variable_data->data,
+        (void*)Ch_GlobalSymbolAddrByName(
+          agent->agent_interp,
+          var_name),
+        size*data_type_size
+        );
+  }
+  agent_variable_data->size = size*data_type_size;
+  
+  /* Make sure that the variable is not already in the agent's list already. */
+  agent_variable_list_Remove(
+      agent->datastate->tasks[task_num]->agent_variable_list, 
+      var_name);
+  agent_variable_list_Add(
+      agent->datastate->tasks[task_num]->agent_variable_list, 
+      agent_variable_data);
+  return 0;
+}
+
   agent_p
 agent_Copy(const agent_p agent)
 {
@@ -322,7 +404,7 @@ agent_RunChScriptThread(void* ChAgent)
 #endif
   MCAgent_t agent;
   mc_platform_p mc_platform;
-  int n;
+  int i,n;
   FILE *TEMP_FILE;
   char *temp_store_file;
   char *ChShellArg[2];
@@ -494,27 +576,27 @@ agent_RunChScriptThread(void* ChAgent)
   /* begin Acl Helper Functions */
   Ch_DeclareFunc(
       agent->agent_interp,
-      "int mc_Acl_SetPerformative(void* acl_message, int performative);",
+      "int mc_AclSetPerformative(void* acl_message, int performative);",
       (ChFuncdl_t)MC_AclSetPerformative_chdl
       );
   Ch_DeclareFunc(
       agent->agent_interp,
-      "int mc_Acl_SetSender(void* acl_message, char* name, char* address);",
+      "int mc_AclSetSender(void* acl_message, char* name, char* address);",
       (ChFuncdl_t)MC_AclSetSender_chdl
       );
   Ch_DeclareFunc(
       agent->agent_interp,
-      "int mc_Acl_AddReceiver(void* acl_message, char* name, char* address);",
+      "int mc_AclAddReceiver(void* acl_message, char* name, char* address);",
       (ChFuncdl_t)MC_AclAddReceiver_chdl
       );
   Ch_DeclareFunc(
       agent->agent_interp,
-      "int mc_Acl_AddReplyTo(void* acl_message, char* name, char* address);",
+      "int mc_AclAddReplyTo(void* acl_message, char* name, char* address);",
       (ChFuncdl_t)MC_AclAddReplyTo_chdl
       );
   Ch_DeclareFunc(
       agent->agent_interp,
-      "int mc_Acl_SetContent(void* acl_message, char* content);",
+      "int mc_AclSetContent(void* acl_message, char* content);",
       (ChFuncdl_t)MC_AclSetContent_chdl
       );
   /* end Acl Helper Functions */
@@ -523,6 +605,16 @@ agent_RunChScriptThread(void* ChAgent)
       "int mc_AddAgent(void* agent);",
       (ChFuncdl_t)MC_AddAgent_chdl
       );
+  Ch_DeclareFunc(
+      agent->agent_interp,
+      "const void* mc_AgentVariableRetrieve(void* agent, const char* var_name, int task_num);",
+      (ChFuncdl_t)MC_AgentVariableRetrieve_chdl
+      );
+  Ch_DeclareFunc(
+      agent->agent_interp,
+      "int mc_AgentVariableSave(void* agent, const char* var_name);",
+      (ChFuncdl_t)MC_AgentVariableSave_chdl
+      );
   Ch_DeclareFunc(
       agent->agent_interp,
       "int mc_Barrier(int id);",
@@ -766,10 +858,11 @@ FIXME: This is silly
   }
   else
   {
+
+#ifndef _WIN32
     /* save the agent to an external file %agentname%%d%d */
     temp_store_file = (char *)malloc(sizeof(char)*30);
 
-#ifndef _WIN32
     strcpy(temp_store_file, "agentchscriptXXXXXX");
     fd = mkstemp(temp_store_file);
     if (fd == -1) {
@@ -781,7 +874,7 @@ FIXME: This is silly
     }
     close(fd);
 #else
-    tmpnam(temp_store_file);
+    temp_store_file = _tempnam(".", "agentchscript");
 #endif
     TEMP_FILE = fopen(temp_store_file, "w");
     
@@ -809,26 +902,39 @@ FIXME: This is silly
 
     /* remove the temp file after its usage */
     remove(temp_store_file);
+#ifndef _WIN32
     free(temp_store_file);
+#endif
   }
 
   /* now add the data element returned from the Ch execution into the agent data structure */
   if(strcmp(agent->datastate->tasks[progress]->var_name, "no-return"))
   {
-    result = agent_return_data_InitializeFromAgent(agent);
+    result = interpreter_variable_data_InitializeFromAgent(agent);
     /* Free old result extracted from XML agent */
-    agent_return_data_Destroy(
+    interpreter_variable_data_Destroy(
         agent->datastate->tasks[progress]->agent_return_data
         );
     /* Replace with new freshly calculated one */
     agent->datastate->tasks[progress]->agent_return_data = 
       result;
   } else {
-    agent_return_data_Destroy(
+    interpreter_variable_data_Destroy(
         agent->datastate->tasks[progress]->agent_return_data );
     agent->datastate->tasks[progress]->agent_return_data = NULL;
   }
 
+  /* This is where we want to save all of the agent variables that the agent
+   * wishes to keep for its trip. */
+  for(i = 0; i < agent->datastate->tasks[progress]->num_saved_variables; i++) {
+    agent_variable_list_Add(
+        agent->datastate->tasks[progress]->agent_variable_list,
+        interpreter_variable_data_Initialize(
+          agent,
+          agent->datastate->tasks[progress]->saved_variables[i] )
+        );
+  }
+
   if (agent->datastate->persistent || 
       agent->datastate->tasks[progress]->persistent ) {
     /* TODO: We need a large while loop here that waits on a condition 
@@ -847,14 +953,25 @@ FIXME: This is silly
     if ((((MCAgent_t)ChAgent)->agent_interp) != NULL) {
       Ch_End(((MCAgent_t)ChAgent)->agent_interp);
     }
+
+    /* Perform some housekeeping regarding agent status */
     if (
-        (agent->datastate->
-         tasks[progress]->agent_return_data == NULL) &&
         (agent->datastate->task_progress ==
          (agent->datastate->number_of_tasks-1))
        ) 
     {
+      /* If the agent is done... */
       ((MCAgent_t) ChAgent)->agent_status = MC_WAIT_FINISHED;
+      /* If _any_ of the tasks have return data, we should generate
+       * a return message. */
+      for(i = 0; 
+          i < agent->datastate->number_of_tasks;
+          i++)
+      {
+        if (agent->datastate->tasks[i]->agent_return_data != NULL) {
+          ((MCAgent_t) ChAgent)->agent_status = MC_WAIT_MESSGSEND;
+        }
+      }
     }
     else {
       ((MCAgent_t) ChAgent)->agent_status = MC_WAIT_MESSGSEND; 
@@ -892,5 +1009,3 @@ FIXME: This is silly
   return 0;
 #endif
 }
-
-
