@@ -9,6 +9,14 @@ agent_mailbox_p agent_mailbox_New(void)
   return mailbox;
 }
 
+agent_mailbox_p agent_mailbox_Copy(agent_mailbox_p src)
+{
+  agent_mailbox_p tmp;
+  tmp = agent_mailbox_New();
+  tmp->mail_queue = mail_queue_Copy(src->mail_queue);
+  return tmp;
+}
+
 int agent_mailbox_Destroy(agent_mailbox_t* mailbox)
 {
   if (mailbox == NULL) return 0;
