@@ -45,6 +45,10 @@ agent_task_New(void)
   } else {
     memset(task, 0, sizeof(agent_task_t)); 
   }
+  task->agent_variable_list = agent_variable_list_New();
+
+  task->saved_variables = NULL;
+  task->num_saved_variables = 0;
 
   return task;
 }
@@ -94,6 +98,7 @@ agent_task_Copy(agent_task_p task)
 int
 agent_task_Destroy( agent_task_p agent_task )
 {
+  int i;
   if(agent_task == NULL) {
     return MC_SUCCESS;
   }
@@ -107,7 +112,16 @@ agent_task_Destroy( agent_task_p agent_task )
     free(agent_task->code_id);
   }
 
-  agent_return_data_Destroy(agent_task->agent_return_data);
+  agent_variable_list_Destroy(agent_task->agent_variable_list);
+
+  if(agent_task->saved_variables != NULL) {
+    for(i = 0; agent_task->saved_variables[i] != NULL; i++) {
+      free(agent_task->saved_variables[i]);
+    }
+    free(agent_task->saved_variables);
+  }
+
+  interpreter_variable_data_Destroy(agent_task->agent_return_data);
 
   free(agent_task);
 
