@@ -31,6 +31,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include "include/connection.h"
 #include "include/mc_error.h"
 
@@ -55,3 +56,26 @@ connection_Destroy(connection_p connection)
   return MC_SUCCESS;
 }
 
+connection_p connection_New(void)
+{
+  connection_p connection;
+  connection = (connection_p)malloc(sizeof(connection_t));
+  if (connection==NULL) {
+    fprintf(stderr, "Memory error. %s:%d\n", __FILE__, __LINE__);
+  }
+  memset(connection, 0, sizeof(connection_t));
+  return connection;
+}
+
+connection_p connection_Copy(connection_p connection)
+{
+  connection_p tmp = connection_New();
+  tmp->connect_id = connection->connect_id;
+  tmp->remote_hostname = strdup(connection->remote_hostname);
+  tmp->addr = connection->addr;
+  tmp->clientfd = connection->clientfd;
+  tmp->serverfd = connection->serverfd;
+
+  return tmp;
+}
+
