@@ -68,6 +68,26 @@ connection_queue_Print(connection_queue_p queue)
   MUTEX_UNLOCK(queue->lock);
   return i;
 }
+
+/* Agent variables */
+#include "include/interpreter_variable_data.h"
+AP_QUEUE_STD_DEFN_TEMPLATE(agent_variable_list, interpreter_variable_data)
+
+AP_QUEUE_SEARCH_TEMPLATE(
+    agent_variable_list,
+    Search,
+    interpreter_variable_data,
+    char*,
+    (!strcmp(node->name, key))
+    )
+
+AP_QUEUE_REMOVE_TEMPLATE(
+    agent_variable_list,
+    Remove,
+    interpreter_variable_data,
+    char*,
+    (!strcmp(node->name, key))
+    )
         
 /* Message Queue */
 AP_QUEUE_STD_DEFN_TEMPLATE(
@@ -113,6 +133,7 @@ message_queue_Print(message_queue_p queue)
 
 
 /* Agent Queue */
+#include "include/agent.h"
 AP_QUEUE_STD_DEFN_TEMPLATE(
     agent_queue,
     agent
