@@ -32,6 +32,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifndef _WIN32
+#include <strings.h>
+#else
+#define strcasecmp(a, b) \
+  _stricmp(a, b)
+#endif
 #include "include/fipa_acl.h"
 #include "include/mc_error.h"
 #include "include/macros.h"
@@ -517,49 +523,49 @@ int fipa_message_type_Parse(
   fipa_word_p word;
   err = fipa_word_Parse(&word, message);
   if (err) return err;
-  if(!strcmp(word->content, "accept-proposal")) {
+  if(!strcasecmp(word->content, "accept-proposal")) {
     *performative = FIPA_ACCEPT_PROPOSAL;
-  } else if (!strcmp(word->content, "agree")) {
+  } else if (!strcasecmp(word->content, "agree")) {
     *performative = FIPA_AGREE;
-  } else if (!strcmp(word->content, "cancel")) {
+  } else if (!strcasecmp(word->content, "cancel")) {
     *performative = FIPA_CANCEL;
-  } else if (!strcmp(word->content, "call-for-proposal")) {
+  } else if (!strcasecmp(word->content, "call-for-proposal")) {
     *performative = FIPA_CALL_FOR_PROPOSAL;
-  } else if (!strcmp(word->content, "confirm")) {
+  } else if (!strcasecmp(word->content, "confirm")) {
     *performative = FIPA_CONFIRM;
-  } else if (!strcmp(word->content, "disconfirm")) {
+  } else if (!strcasecmp(word->content, "disconfirm")) {
     *performative = FIPA_DISCONFIRM;
-  } else if (!strcmp(word->content, "failure")) {
+  } else if (!strcasecmp(word->content, "failure")) {
     *performative = FIPA_FAILURE;
-  } else if (!strcmp(word->content, "inform")) {
+  } else if (!strcasecmp(word->content, "inform")) {
     *performative = FIPA_INFORM;
-  } else if (!strcmp(word->content, "inform-if")) {
+  } else if (!strcasecmp(word->content, "inform-if")) {
     *performative = FIPA_INFORM_IF;
-  } else if (!strcmp(word->content, "inform-ref")) {
+  } else if (!strcasecmp(word->content, "inform-ref")) {
     *performative = FIPA_INFORM_REF;
-  } else if (!strcmp(word->content, "not-understood")) {
+  } else if (!strcasecmp(word->content, "not-understood")) {
     *performative = FIPA_NOT_UNDERSTOOD;
-  } else if (!strcmp(word->content, "propogate")) {
+  } else if (!strcasecmp(word->content, "propogate")) {
     *performative = FIPA_PROPOGATE;
-  } else if (!strcmp(word->content, "propose")) {
+  } else if (!strcasecmp(word->content, "propose")) {
     *performative = FIPA_PROPOSE;
-  } else if (!strcmp(word->content, "proxy")) {
+  } else if (!strcasecmp(word->content, "proxy")) {
     *performative = FIPA_PROXY;
-  } else if (!strcmp(word->content, "query-if")) {
+  } else if (!strcasecmp(word->content, "query-if")) {
     *performative = FIPA_QUERY_IF;
-  } else if (!strcmp(word->content, "query-ref")) {
+  } else if (!strcasecmp(word->content, "query-ref")) {
     *performative = FIPA_QUERY_REF;
-  } else if (!strcmp(word->content, "refuse")) {
+  } else if (!strcasecmp(word->content, "refuse")) {
     *performative = FIPA_REFUSE;
-  } else if (!strcmp(word->content, "reject-proposal")) {
+  } else if (!strcasecmp(word->content, "reject-proposal")) {
     *performative = FIPA_REJECT_PROPOSAL;
-  } else if (!strcmp(word->content, "request")) {
+  } else if (!strcasecmp(word->content, "request")) {
     *performative = FIPA_REQUEST;
-  } else if (!strcmp(word->content, "request-when")) {
+  } else if (!strcasecmp(word->content, "request-when")) {
     *performative = FIPA_REQUEST_WHEN;
-  } else if (!strcmp(word->content, "request-whenever")) {
+  } else if (!strcasecmp(word->content, "request-whenever")) {
     *performative = FIPA_REQUEST_WHENEVER;
-  } else if (!strcmp(word->content, "subscribe")) {
+  } else if (!strcasecmp(word->content, "subscribe")) {
     *performative = FIPA_SUBSCRIBE;
   } else {
     fprintf(stderr, "Unknown performative: '%s'. %s:%d\n",
