@@ -102,12 +102,18 @@ typedef agent_t* MCAgent_t;
 typedef agent_t* agent_p;
 #endif
 
+int 
+agent_AddPersistentVariable(agent_p agent, int task_num, const char* var_name);
+
 agent_p
 agent_New(void);
 
 agent_p
 agent_Copy(const agent_p agent);
 
+  int
+agent_Destroy(agent_p agent);
+
 agent_p
 agent_Initialize(
     struct mc_platform_s* mc_platform,
