@@ -73,6 +73,12 @@ MC_AclSetContent_chdl(void *varg);
 EXPORTCH int
 MC_AddAgent_chdl(void *varg);
 
+EXPORTCH const void*
+MC_AgentVariableRetrieve_chdl(void* varg);
+
+EXPORTCH int
+MC_AgentVariableSave_chdl(void *varg);
+
 EXPORTCH int
 MC_Barrier_chdl(void *varg);
 
