@@ -31,7 +31,8 @@
 
 #ifndef _AGENT_TASK_H_
 #define _AGENT_TASK_H_
-#include "agent_return_data.h"
+#include "interpreter_variable_data.h"
+#include "data_structures.h"
 
 typedef struct agent_task_s{
  
@@ -42,16 +43,15 @@ typedef struct agent_task_s{
   int persistent;
   int init_agent_status;
 
-  char *var_name;
-  char *server_name;
-  char *code_id;
+  char* var_name;
+  char* server_name;
+  char* code_id;
 
   /* data elements */
-  agent_return_data_t *agent_return_data;
-
-  /* code for any of the code that is ascocitated with the tasks */
-  /* char* task_code_elements; */
-  /* NOTE: All agent code is in the agent struct now. */
+  interpreter_variable_data_t* agent_return_data;
+  struct agent_variable_list_s* agent_variable_list;
+  char** saved_variables;
+  int num_saved_variables;
 
 } agent_task_t;
 
