@@ -47,9 +47,10 @@ typedef name##_t* name##_p; \
   \
 name##_p name##_New( void ); \
 int name##_Destroy( name##_p name ); \
-int name##_Add( name##_p name, node_type##_t* node ); \
-node_type##_t* name##_Pop( name##_p name ); \
-node_type##_t* name##_SearchIndex( name##_p name, int index ); \
+int name##_Add( name##_p name, struct node_type##_s* node ); \
+name##_p name##_Copy(name##_p name); \
+struct node_type##_s* name##_Pop( name##_p name ); \
+struct node_type##_s* name##_SearchIndex( name##_p name, int index ); \
 int name##_RemoveIndex(name##_p name, int index); 
 
 #define AP_QUEUE_GENERIC_DECL_TEMPLATE(name, func_name, return_type, search_type) \
@@ -72,7 +73,7 @@ name##_p name##_New( void ) \
   \
 int name##_Destroy( name##_p name ) \
 { \
-  node_type##_t* node; \
+  struct node_type##_s* node; \
   while ((node = name##_Pop(name)) != NULL) { \
     node_type##_Destroy(node); \
   } \
@@ -85,6 +86,41 @@ int name##_Destroy( name##_p name ) \
   return 0; \
 } \
   \
+name##_p name##_Copy(name##_p name) \
+{ \
+  int i;\
+  name##_p temp; \
+  struct node_type##_s* temp_node; \
+  temp = (name##_p)malloc(sizeof(name##_t)); \
+  if (temp == NULL) { \
+    fprintf(stderr, "Memory Error at %s:%d\n", __FILE__, __LINE__); \
+    exit(0); \
+  } \
+  temp->size = 0; \
+  temp->list = ListInitialize(); \
+  temp->lock = (MUTEX_T*)malloc(sizeof(MUTEX_T)); \
+  if (temp->lock == NULL) { \
+    fprintf(stderr, "Memory Error at %s:%d\n", __FILE__, __LINE__); \
+    exit(0); \
+  } \
+  temp->cond = (COND_T*)malloc(sizeof(COND_T)); \
+  if(temp->cond == NULL) { \
+    fprintf(stderr, "Memory Error at %s:%d\n", __FILE__, __LINE__); \
+    exit(0); \
+  } \
+  MUTEX_INIT(temp->lock); \
+  COND_INIT(temp->cond); \
+  for( \
+      i = 0; \
+      (temp_node = name##_SearchIndex(name, i)) != NULL; \
+      i++ \
+     ) \
+  { \
+    name##_Add(temp, node_type##_Copy(temp_node)); \
+  } \
+  return temp; \
+} \
+  \
 int name##_Add( name##_p name, node_type##_t* node ) \
 { \
   MUTEX_LOCK(name->lock); \
@@ -97,7 +133,7 @@ int name##_Add( name##_p name, node_type##_t* node ) \
   \
 node_type##_t* name##_Pop( name##_p name ) \
 { \
-  node_type##_t* ret; \
+  struct node_type##_s* ret; \
   MUTEX_LOCK(name->lock); \
   if (name->size <= 0) { \
     MUTEX_UNLOCK(name->lock); \
@@ -110,9 +146,9 @@ node_type##_t* name##_Pop( name##_p name ) \
   return ret; \
 } \
   \
-node_type##_t* name##_SearchIndex( name##_p name, int index ) \
+struct node_type##_s* name##_SearchIndex( name##_p name, int index ) \
 { \
-  node_type##_t* node; \
+  struct node_type##_s* node; \
   MUTEX_LOCK(name->lock); \
   node = (node_type##_t*)ListSearch(name->list, index); \
   MUTEX_UNLOCK(name->lock); \
@@ -121,7 +157,7 @@ node_type##_t* name##_SearchIndex( name##_p name, int index ) \
   \
 int name##_RemoveIndex( name##_p name, int index ) \
 { \
-  node_type##_t* node; \
+  struct node_type##_s* node; \
   MUTEX_LOCK(name->lock); \
   node = ListDelete(name->list, index); \
   node_type##_Destroy(node); \
@@ -132,11 +168,11 @@ int name##_RemoveIndex( name##_p name, int index ) \
 
 #define AP_QUEUE_SEARCH_TEMPLATE( name, func_name, node_type, \
     search_type, search_expression ) \
-node_type##_t* name##_##func_name( name##_p name, const search_type key ) \
+struct node_type##_s* name##_##func_name( name##_p name, const search_type key ) \
 { \
   listNode_t* parsenode; \
-  node_type##_t* node; \
-  node_type##_t* ret = NULL; \
+  struct node_type##_s* node; \
+  struct node_type##_s* ret = NULL; \
   node = NULL; \
   \
   MUTEX_LOCK(name->lock); \
@@ -165,8 +201,8 @@ node_type##_t* name##_##func_name( name##_p name, const search_type key ) \
 int name##_##func_name( name##_p name, const search_type key ) \
 { \
   int err_code = MC_ERR_NOT_FOUND; \
-  listNode_t* parsenode; \
-  node_type##_t* node; \
+  struct listNode_s* parsenode; \
+  struct node_type##_s* node; \
   node = NULL; \
   \
   MUTEX_LOCK(name->lock); \
