@@ -57,9 +57,15 @@ typedef struct connection_s {
 typedef connection_t* connection_p;
 
 /* glocal functions - basic */
+connection_p 
+connection_New(void);
+
 int 
 connection_Destroy(connection_p connection);
 
+connection_p 
+connection_Copy(connection_p connection);
+
 void connection_Close(connection_p connection);
 void connection_SetHosts(connection_p connection, char* remote_host, char *local_host);
 
