@@ -73,7 +73,7 @@ int
 message_queue_Print(message_queue_p queue);
 
 /* Agent Queue */
-#include"agent.h"
+struct agent_s;
 AP_QUEUE_DECL_TEMPLATE(
     agent_queue,
     agent
@@ -82,14 +82,14 @@ AP_QUEUE_DECL_TEMPLATE(
 AP_QUEUE_GENERIC_DECL_TEMPLATE(
     agent_queue,
     Search,
-    agent_t*,
+    struct agent_s*,
     int
     )
 
 AP_QUEUE_GENERIC_DECL_TEMPLATE(
     agent_queue,
     SearchName,
-    agent_t*,
+    struct agent_s*,
     char*
     )
 
@@ -110,6 +110,24 @@ AP_QUEUE_GENERIC_DECL_TEMPLATE(
 int
 agent_queue_Print(agent_queue_p queue);
 
+/* Agent variables */
+#include "interpreter_variable_data.h"
+AP_QUEUE_DECL_TEMPLATE(
+    agent_variable_list,
+    interpreter_variable_data)
+
+AP_QUEUE_GENERIC_DECL_TEMPLATE(
+    agent_variable_list,
+    Search,
+    interpreter_variable_data_t*,
+    char* )
+
+AP_QUEUE_GENERIC_DECL_TEMPLATE(
+    agent_variable_list,
+    Remove,
+    int,
+    char* )
+
 /* Mail Queue */
 AP_QUEUE_DECL_TEMPLATE(
     mail_queue,
