@@ -39,6 +39,11 @@
  * \section intro_sec Introduction
  * Welcome to the MobileC documentation. For a quick user-api reference, please
  * refer to the file libmc.h.
+ *
+ * This documentation is provided supplementary to the main Mobile-C User Guide. 
+ * The official user guide may be obtained at http://www.mobilec.org , and should
+ * also be included with your Mobile-C Distribution package in the file
+ * docs/mobilec.pdf .
  */
 
 /* Filename: libmc.h */
@@ -192,13 +197,13 @@ typedef enum MC_SteerCommand_e {
  * \sa        MC_WaitSignal(), MC_ResetSignal
  */
 enum MC_Signal_e{
-    MC_NO_SIGNAL      =0x00,
-    MC_RECV_CONNECTION=0x01,
-    MC_RECV_MESSAGE   =0x02,
-    MC_RECV_AGENT     =0x04,
-    MC_RECV_RETURN    =0x08,
-    MC_EXEC_AGENT     =0x0F,
-    MC_ALL_SIGNALS    =0x10
+    MC_NO_SIGNAL      =0x00, /*!< No signal hass been received */
+    MC_RECV_CONNECTION=0x01, /*!< A Connection has been received */
+    MC_RECV_MESSAGE   =0x02, /*!< A Message has been received from a connection */
+    MC_RECV_AGENT     =0x04, /*!< An normal agent has been parsed from a message */
+    MC_RECV_RETURN    =0x08, /*!< A return agent has been parsed from a message */
+    MC_EXEC_AGENT     =0x0F, /*!< A normal agent has been executed. */
+    MC_ALL_SIGNALS    =0x10  /*!< Catch any of the above signals */
 };
 
 /**
@@ -329,6 +334,7 @@ EXPORTMC extern struct fipa_acl_message_s* MC_AclWaitRetrieve(MCAgent_t agent);
 
 /* ACL Helper Functions Here */
 enum fipa_performative_e;
+
 EXPORTMC int MC_AclSetPerformative(
     struct fipa_acl_message_s* acl,
     enum fipa_performative_e performative );
@@ -367,6 +373,47 @@ EXPORTMC extern int MC_AddAgent(
     MCAgency_t attr, 
     MCAgent_t agent);
 
+/**
+ * \brief           Retrieve a pointer to a previously saved variable.
+ *
+ * \param agent     A MobileC agent.
+ * \param var_name  The name of the saved variable that has previously
+ *                  been saved.
+ * \param task_num  The previous completed task from which to retrieve
+ *                  the saved variable.
+ *
+ * \return          A pointer to the data on success or NULL on failure.
+ * \section         Examples
+ * The following example demonstrates usage of MC_AgentVariableRetrieve() 
+ * from agent space.
+ * \include agent_saved_variables_example/test1.xml
+ */
+extern const void* 
+MC_AgentVariableRetrieve(
+    MCAgent_t agent,
+    const char* var_name,
+    int task_num);
+
+/**
+ * \brief           Mark an agent variable for saving
+ *
+ * \param agent     A MobileC agent.
+ * \param var_name  The name of the variable to mark for saving.
+ *
+ * \return          0 on success, non-zero on failure.
+ * \sa test1.xml
+ * \section         Examples
+ * 
+ * See agent_saved_variables_example/test1.xml for an example of usage of
+ * this api function.
+ * \section         Examples
+ * The following example demonstrates usage of MC_AgentVariableSave() 
+ * from agent space.
+ * \include agent_saved_variables_example/test1.xml
+ */
+extern int 
+MC_AgentVariableSave(MCAgent_t agent, const char* var_name);
+
 /**
  * \brief           Find and delete an initialized MobileC Barrier
  *
@@ -428,6 +475,20 @@ EXPORTMC extern int MC_CallAgentFunc(
         void* varg
         );
 
+
+/**
+ * \brief           Compose a new agent dynamically without using a prewritten XML file.
+ *
+ * \param name      The desired name of the new agent.
+ * \param home      The home of the new agent.
+ * \param owner     The owner of the new agent.
+ * \param code      The agent code
+ * \param return_var_name The name of the agent's return variable. Set to "no-return" if no return variable is desired.
+ * \param server    The target destination of the agent.
+ * \param persitent A flag indicating whether or not the agent should be persistent. A value of '1' indicates persistence, while a value of '0' indicates default non-persistent behaviour.
+ *
+ * \return          This function returns a valid MCAgent_t structure on success or NULL on failure.
+ */
 MCAgent_t
 MC_ComposeAgent(
     const char* name,
@@ -684,6 +745,16 @@ EXPORTMC extern int MC_InitializeAgencyOptions(struct MCAgencyOptions_s* options
  */
  EXPORTMC extern int MC_MainLoop(MCAgency_t attr);
 
+/**
+ * \brief           Load an agent from a file into an agency
+ *
+ * \param agency    A valid and running Mobile-C agency
+ * \param filename  Filename containing the agent to load
+ *
+ * \return          0 on success, non-zero on failure.
+ */
+EXPORTMC extern int MC_LoadAgentFromFile(MCAgency_t attr, const char* filename);
+
 /**
  * \brief           Locks a MobileC synchronization variable as a mutex
  *
@@ -1035,5 +1106,6 @@ EXPORTMC extern int MC_WaitSignal(MCAgency_t attr, int signals);
 #ifdef __cplusplus
 }
 #endif
+
 #endif /* _MOBILEC_H_ */
 
