@@ -457,30 +457,30 @@ WaitForSingleObject( *(queue->touched_signal), INFINITE )
 #define CH_DATATYPE_SIZE(type, size)                            \
   switch(type) {                                              \
     case CH_CHARTYPE:                                       \
-                                                            size = sizeof(char);                                \
+      size = sizeof(char);                                \
     break;                                              \
     case CH_INTTYPE:                                        \
-                                                            size = sizeof(int);                                 \
+      size = sizeof(int);                                 \
     break;                                              \
     case CH_UINTTYPE:                                       \
-                                                            size = sizeof(unsigned int);                        \
+      size = sizeof(unsigned int);                        \
     break;                                              \
     case CH_SHORTTYPE:                                      \
-                                                            size = sizeof(short);                               \
+      size = sizeof(short);                               \
     break;                                              \
     case CH_USHORTTYPE:                                     \
-                                                            size = sizeof(unsigned short);                      \
+      size = sizeof(unsigned short);                      \
     break;                                              \
     case CH_FLOATTYPE:                                      \
-                                                            size = sizeof(float);                               \
+      size = sizeof(float);                               \
     break;                                              \
     case CH_DOUBLETYPE:                                     \
-                                                            size = sizeof(double);                              \
+      size = sizeof(double);                              \
     break;                                              \
     default:                                                \
-                                                            fprintf(stderr, "Unknown data type: %d at %s:%d",   \
-                                                                type, __FILE__, __LINE__);                  \
-    size=0;                                             \
+      fprintf(stderr, "Unknown data type: %d at %s:%d",   \
+        type, __FILE__, __LINE__);                  \
+      size=0;                                             \
   }
 
 /* Macro for converting a CH_Type_t into a string */
@@ -519,6 +519,9 @@ WaitForSingleObject( *(queue->touched_signal), INFINITE )
  * a string. */
 #define CH_DATATYPE_VALUE_STRING(type, string, p) \
   switch(type) {                                      \
+    case CH_CHARTYPE:                                 \
+      sprintf(string, "%c", *((char*)p));             \
+      break;                                          \
     case CH_INTTYPE:                                \
       sprintf(string, "%d", *((int*)p));          \
     break;                                      \
@@ -556,6 +559,8 @@ WaitForSingleObject( *(queue->touched_signal), INFINITE )
     type = CH_SHORTTYPE;                            \
   } else if (!strcmp(string, "unsigned short")) {     \
     type = CH_USHORTTYPE;                           \
+  } else if (!strcmp(string, "char")) {             \
+    type = CH_CHARTYPE;                             \
   } else {                                            \
     fprintf(stderr,                                 \
         "Unsupported data type: %d %s:%d\n",    \
