@@ -96,6 +96,7 @@ enum http_performative_e
   HTTP_TRACE,
   HTTP_OPTIONS,
   HTTP_CONNECT,
+  HTTP_RESPONSE,
   HTTP_NUM_PERFORMATIVES
 };
 
@@ -125,6 +126,10 @@ typedef struct mtp_http_s
   char* cache_control;
   char* mime_version;
 
+  /* The following are only valid if the message is a response */
+  int response_code;
+  char* response_string;
+
   /* The message may be a multi-part message */
   /* FIXME: We don't support nested multi-part messages for now */
   int message_parts;
@@ -133,6 +138,8 @@ typedef struct mtp_http_s
 } mtp_http_t;
 typedef mtp_http_t* mtp_http_p;
 
+#define SOCKET_INPUT_SIZE 4096
+
 /**
  * \brief         Parse an html expression
  *
