@@ -324,6 +324,44 @@ MC_AddAgent(MCAgency_t attr, MCAgent_t agent) /*{{{*/
   return 0;
 } /*}}}*/
 
+const void* MC_AgentVariableRetrieve(MCAgent_t agent, const char* var_name, int task_num)
+{
+  interpreter_variable_data_t* interp_var;
+
+  if (task_num >= agent->datastate->task_progress) {
+    return NULL;
+  }
+
+  interp_var = agent_variable_list_Search(
+      agent->datastate->tasks[task_num]->agent_variable_list,
+      var_name );
+  if (interp_var == NULL) {
+    return NULL;
+  }
+
+  return interp_var->data;
+}
+
+int MC_AgentVariableSave(MCAgent_t agent, const char* var_name)
+{
+  int current_task = agent->datastate->task_progress;
+  const int default_num_vars = 50;
+  agent_task_p task = agent->datastate->tasks[current_task];
+
+  if(task->num_saved_variables == 0) {
+    task->saved_variables = (char**)malloc(sizeof(char*)*default_num_vars);
+    memset(task->saved_variables, 0, sizeof(char*)*default_num_vars);
+  }
+  task->saved_variables[task->num_saved_variables] = strdup(var_name);
+  if(task->saved_variables[task->num_saved_variables] == NULL) {
+    fprintf(stderr, "Memory error. %s:%d\n", __FILE__, __LINE__);
+    return MC_ERR_MEMORY;
+  }
+  task->num_saved_variables++;
+
+  return 0;
+}
+
 int 
 MC_Barrier(MCAgency_t attr, int id) /*{{{*/
 {
@@ -627,25 +665,37 @@ MC_End(MCAgency_t agency) /*{{{*/
   agency->mc_platform->quit = 1;
   MUTEX_UNLOCK(agency->mc_platform->quit_lock);
   COND_SIGNAL(agency->mc_platform->df->request_list->cond);
-  COND_SIGNAL(agency->mc_platform->ams->runflag_cond);
-  COND_SIGNAL(agency->mc_platform->connection_queue->cond);
-  COND_SIGNAL(agency->mc_platform->message_queue->cond);
-  if( GET_THREAD_MODE( agency->threads, MC_THREAD_ACC)) {
-    THREAD_CANCEL( agency->mc_platform->acc->listen_thread );
-  }
+
+  /* Stop the command prompt */
   if( GET_THREAD_MODE( agency->threads, MC_THREAD_CP)) {
     THREAD_CANCEL( agency->mc_platform->cmd_prompt->thread );
   }
 
-  if( GET_THREAD_MODE( agency->threads, MC_THREAD_DF)) {
-    THREAD_JOIN(agency->mc_platform->df->thread);
+  /* Stop the listen thread */
+  if( GET_THREAD_MODE( agency->threads, MC_THREAD_ACC)) {
+    THREAD_CANCEL( agency->mc_platform->acc->listen_thread );
   }
+
+  /* Stop the connection queue */
+  COND_SIGNAL(agency->mc_platform->connection_queue->cond);
+  if( GET_THREAD_MODE( agency->threads, MC_THREAD_ACC)) {
+    THREAD_JOIN(agency->mc_platform->acc->thread);
+  }
+
+  /* Stop the message queue */
+  if( GET_THREAD_MODE( agency->threads, MC_THREAD_ACC)) {
+    THREAD_JOIN(agency->mc_platform->acc->message_handler_thread);
+  }
+
+  /* Stop the AMS */
+  COND_SIGNAL(agency->mc_platform->ams->runflag_cond);
   if( GET_THREAD_MODE( agency->threads, MC_THREAD_AMS)) {
     THREAD_JOIN(agency->mc_platform->ams->thread);
   }
-  if( GET_THREAD_MODE( agency->threads, MC_THREAD_ACC)) {
-    THREAD_JOIN(agency->mc_platform->acc->thread);
-    THREAD_JOIN(agency->mc_platform->acc->message_handler_thread);
+
+
+  if( GET_THREAD_MODE( agency->threads, MC_THREAD_DF)) {
+    THREAD_JOIN(agency->mc_platform->df->thread);
   }
 
   mc_platform_Destroy(agency->mc_platform);
@@ -799,7 +849,7 @@ MC_GetAgentReturnData( /*{{{*/
   memcpy(
       *data,
       agent->datastate->tasks[task_num]->
-      agent_return_data->return_data,
+      agent_return_data->data,
       size * num_elements
       );
   *dim = agent->datastate->tasks[task_num]->agent_return_data->array_dim;
@@ -855,10 +905,12 @@ MC_GetAllAgents(MCAgency_t attr, MCAgent_t **agents, int* num_agents) /*{{{*/
   index = 0;
   while
     (
-     (*agents)[index] = agent_queue_SearchIndex
      (
-      attr->mc_platform->agent_queue,
-      index
+      (*agents)[index] = agent_queue_SearchIndex
+      (
+       attr->mc_platform->agent_queue,
+       index
+      )
      )
     )
     {
@@ -869,8 +921,6 @@ MC_GetAllAgents(MCAgency_t attr, MCAgent_t **agents, int* num_agents) /*{{{*/
   return 0;
 } /*}}}*/
 
-
-
 EXPORTMC int
 MC_HaltAgency(MCAgency_t attr) /*{{{*/
 {
@@ -952,6 +1002,76 @@ MC_InitializeAgencyOptions(struct MCAgencyOptions_s* options) /*{{{*/
   return 0;
 } /*}}}*/
 
+EXPORTMC int
+MC_LoadAgentFromFile(MCAgency_t attr, const char* filename)
+{
+  struct stat filestat;
+  char *buf;
+  FILE *fp;
+  message_p message;
+  extern mc_platform_p g_mc_platform;
+  buf = NULL;
+  filestat.st_size = 0;
+  stat(filename, &filestat);
+  if (filestat.st_size != 0 ) {
+    buf = malloc( sizeof(char) * (filestat.st_size+1) );
+    memset(buf, 0, filestat.st_size+1);
+  } else {
+    fprintf(stderr, "Error: File %s not found.\n", filename);
+    return 1;
+  }
+
+  fp = fopen(filename, "r");
+  fread((void*)buf, filestat.st_size, 1, fp);
+  fclose(fp);
+
+  message = message_New();
+  if ( 
+      message_InitializeFromString (
+      attr->mc_platform,
+      message,
+      buf,
+      "",
+      5050,
+      "ams"
+      )
+     )
+  {
+    message_Destroy(message);
+  }
+  free(message->to_address);
+  message->to_address = NULL;
+  message->xml_root = mxmlLoadString(
+      NULL,
+      buf,
+      NULL );
+  if (message->xml_root == NULL) {
+    fprintf(stderr, "Error loading agent. %s:%d\n", __FILE__, __LINE__);
+    message_Destroy(message);
+    return 1;
+  }
+  message->xml_payload = mxmlFindElement(
+      message->xml_root,
+      message->xml_root,
+      "MOBILE_AGENT",
+      NULL,
+      NULL,
+      MXML_DESCEND );
+  if(message->xml_payload == NULL) {
+    fprintf(stderr, "Error loading agent: <MOBILE_AGENT> tag not found. %s:%d\n",
+        __FILE__, __LINE__ );
+    message_Destroy(message);
+    return 1;
+  }
+
+  message_queue_Add
+    (
+     attr->mc_platform->message_queue,
+     message
+    );
+  return 0;
+}
+
 EXPORTMC int 
 MC_MutexLock(MCAgency_t attr, int id) /*{{{*/
 {
@@ -1809,6 +1929,52 @@ MC_AddAgent_chdl(void *varg) /*{{{*/
   return retval;
 } /*}}}*/
 
+/* MC_AgentVariableRetrieve */
+EXPORTCH const void*
+MC_AgentVariableRetrieve_chdl(void* varg)
+{
+  void* retval;
+  MCAgent_t agent;
+  const char* var_name;
+  int task_num;
+
+  ChInterp_t interp;
+  ChVaList_t ap;
+
+  Ch_VaStart(interp, ap, varg);
+
+  agent = Ch_VaArg(interp, ap, MCAgent_t);
+  var_name = Ch_VaArg(interp, ap, const char* );
+  task_num = Ch_VaArg(interp, ap, int);
+
+  retval = MC_AgentVariableRetrieve(agent, var_name, task_num);
+
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
+/* MC_AgentVariableSave */
+EXPORTCH int
+MC_AgentVariableSave_chdl(void *varg)
+{
+  int retval;
+  MCAgent_t agent;
+  const char* var_name;
+
+  ChInterp_t interp;
+  ChVaList_t ap;
+
+  Ch_VaStart(interp, ap, varg);
+
+  agent = Ch_VaArg(interp, ap, MCAgent_t);
+  var_name = Ch_VaArg(interp, ap, const char*);
+
+  retval = MC_AgentVariableSave(agent, var_name);
+
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
 /* MC_CallAgentFunc */
 EXPORTCH int
 MC_CallAgentFunc_chdl(void *varg) /*{{{*/
