@@ -157,6 +157,9 @@ mc_platform_Initialize(MCAgency_t agency)
   mc_platform->quit_lock = (MUTEX_T*)malloc(sizeof(MUTEX_T));
   MUTEX_INIT(mc_platform->quit_lock);
 
+  /* Initialize MC_signal flag */
+  mc_platform->MC_signal = MC_NO_SIGNAL;
+
   /* Allocate and init giant lock */
   mc_platform->giant = 1;
   mc_platform->giant_lock = (MUTEX_T*)malloc(sizeof(MUTEX_T));
@@ -179,17 +182,46 @@ mc_platform_Initialize(MCAgency_t agency)
     mc_platform->security_manager = asm_Initialize(mc_platform);
   }
 #endif
-  if (GET_THREAD_MODE(agency->threads, MC_THREAD_DF))
+  if (GET_THREAD_MODE(agency->threads, MC_THREAD_DF)) {
     df_Start(mc_platform);
-  if (GET_THREAD_MODE(agency->threads, MC_THREAD_AMS))
+    MUTEX_LOCK(mc_platform->df->waiting_lock);
+    /* Wait for the thread to fully initialize before continuing */
+    while(mc_platform->df->waiting == 0) {
+      COND_WAIT(mc_platform->df->waiting_cond, mc_platform->df->waiting_lock);
+    }
+    MUTEX_UNLOCK(mc_platform->df->waiting_lock);
+  }
+  if (GET_THREAD_MODE(agency->threads, MC_THREAD_AMS)) {
     ams_Start(mc_platform);
-  if (GET_THREAD_MODE(agency->threads, MC_THREAD_ACC))
+    MUTEX_LOCK(mc_platform->ams->waiting_lock);
+    /* Wait for the thread to fully initialize before continuing */
+    while(mc_platform->ams->waiting == 0) {
+      COND_WAIT(mc_platform->ams->waiting_cond, mc_platform->ams->waiting_lock);
+    }
+    MUTEX_UNLOCK(mc_platform->ams->waiting_lock);
+  }
+  if (GET_THREAD_MODE(agency->threads, MC_THREAD_ACC)) {
     acc_Start(mc_platform);
+    MUTEX_LOCK(mc_platform->acc->waiting_lock);
+    /* Wait for the thread to fully initialize before continuing */
+    while(mc_platform->acc->waiting == 0) {
+      COND_WAIT(mc_platform->acc->waiting_cond, mc_platform->acc->waiting_lock);
+    }
+    MUTEX_UNLOCK(mc_platform->acc->waiting_lock);
+  }
   if (GET_THREAD_MODE(agency->threads, MC_THREAD_CP))
     cmd_prompt_Start(mc_platform);
 #ifdef MC_SECURITY
   if (agency->enable_security) {
     asm_Start(mc_platform);
+    MUTEX_LOCK(mc_platform->security_manager->waiting_lock);
+    /* Wait for the thread to fully initialize before continuing */
+    while(mc_platform->security_manager->waiting == 0) {
+      COND_WAIT(
+          mc_platform->security_manager->waiting_cond,
+          mc_platform->security_manager->waiting_lock);
+    }
+    MUTEX_UNLOCK(mc_platform->security_manager->waiting_lock);
   }
 #endif
 
