@@ -41,6 +41,7 @@
 #include "winconfig.h"
 #endif
 #include <mxml.h>
+#include "include/libmc.h"
 #include "include/agent.h"
 #include "include/mc_platform.h"
 #include "include/message.h"
@@ -71,7 +72,7 @@ int message_Decrypt(message_p message, asm_node_p asm_node)
     (
      message->xml_root,
      message->xml_root,
-     "GAF_MESSAGE",
+     "MOBILEC_MESSAGE",
      NULL,
      NULL,
      MXML_DESCEND
@@ -227,7 +228,7 @@ message_Encrypt(message_p message, asm_node_p asm_node)
     (
      message->xml_root,
      message->xml_root,
-     "GAF_MESSAGE",
+     "MOBILEC_MESSAGE",
      NULL,
      NULL,
      MXML_DESCEND
@@ -302,6 +303,14 @@ message_New(void)
   return message;
 }
 
+message_p
+message_Copy(message_p src)
+{
+  fprintf(stderr, "FIXME: message_Copy() is not implemented yet. %s:%d\n",
+      __FILE__, __LINE__);
+  return NULL;
+}
+
 int
 message_InitializeFromAgent(
     mc_platform_p mc_platform,
@@ -575,13 +584,15 @@ message_InitializeFromString(
 
   /* Set up message->addr */
   message->addr = (struct sockaddr_in*)malloc(sizeof(struct sockaddr_in));
-  if((host = gethostbyname(destination_host)))
-  {
-    memcpy(&(message->addr->sin_addr), host->h_addr, host->h_length);
-    message->addr->sin_port = htons(destination_port);
-  } else {
-    fprintf(stderr, "Warning: Host not found: %s:%d  %s:%d",
-        destination_host, destination_port, __FILE__, __LINE__ );
+  if (destination_host != NULL && strlen(destination_host)!= 0) {
+    if((host = gethostbyname(destination_host)))
+    {
+      memcpy(&(message->addr->sin_addr), host->h_addr, host->h_length);
+      message->addr->sin_port = htons(destination_port);
+    } else {
+      fprintf(stderr, "Warning: Host not found: %s:%d  %s:%d",
+          destination_host, destination_port, __FILE__, __LINE__ );
+    }
   }
 
   return MC_SUCCESS;
@@ -590,7 +601,6 @@ message_InitializeFromString(
   int
 message_Destroy(message_p message)
 {
-  int i;
   if (message == NULL) {
     return MC_SUCCESS;
   }
@@ -629,6 +639,9 @@ message_Destroy(message_p message)
   int
 message_Send(message_p message)
 {
+  char *buffer;
+  mtp_http_t* mtp_http;
+  int n;
 #ifndef _WIN32
   int skt;
   struct sockaddr_in sktin;
@@ -690,7 +703,7 @@ message_Send(message_p message)
     return MC_ERR_CONNECT;
   }
   /* now send the string */
-  if(send(skt, message->message_body, strlen(message->message_body)+1, 0) < 0) 
+  if(send(skt, message->message_body, strlen(message->message_body), 0) < 0) 
   {
     fprintf(stderr, "cannot write to socket %s:%d\n",
         __FILE__, __LINE__);
@@ -702,6 +715,35 @@ message_Send(message_p message)
     free(buf);
     return MC_ERR_SEND;
   }
+  /* Now we should receive an HTTP response */
+  buffer = (char*) malloc(sizeof(char) * (SOCKET_INPUT_SIZE + 1));
+  CHECK_NULL(buffer, exit(0););
+  mtp_http = mtp_http_New();
+#ifndef _WIN32
+  n = recvfrom(skt,
+      (void *) buffer,
+      (size_t) sizeof(char)*SOCKET_INPUT_SIZE,
+      0,
+      (struct sockaddr *) 0,
+      (socklen_t *) 0);
+#else
+  n = recvfrom(skt,
+      (void *) buffer,
+      (size_t) sizeof(char)*SOCKET_INPUT_SIZE,
+      0,
+      (struct sockaddr *) 0,
+      0);
+#endif
+  if( mtp_http_Parse(mtp_http, buffer) ) {
+    fprintf(stderr, "http parsing error: Response expected. %s:%d\n",
+        __FILE__, __LINE__);
+    fprintf(stderr, "Received message was:\n%s\n", buffer);
+  }
+  if (mtp_http->response_code != 200) {
+    fprintf(stderr, "Warning: remote http server responded: %d %s\n",
+        mtp_http->response_code, mtp_http->response_string );
+  }
+
 #ifndef _WIN32
   close(skt);
 #else
