@@ -34,6 +34,7 @@
 #include <string.h>
 #include <time.h>
 #ifndef _WIN32
+#include <unistd.h>
 #include "config.h"
 #else
 #include "winconfig.h"
@@ -45,7 +46,6 @@
 #include "include/message.h"
 #include "include/dynstring.h"
 
-#define SOCKET_INPUT_SIZE 4096
 int
 mtp_http_Destroy(mtp_http_p http)
 {
@@ -129,8 +129,57 @@ mtp_http_InitializeFromConnection
       free(buffer);
       return MC_ERR_CONNECT;
     } 
-    else if (n == 0) {
+    else if (n == 0 || n < SOCKET_INPUT_SIZE) {
+      if (n != 0) {
+        buffer[n] = '\0';
+        strcat(message_string, buffer);
+      }
       free(buffer);
+      if(mtp_http_Parse(http, message_string)) {
+        /* Reply with an HTTP error code */
+        buffer = malloc
+          (
+           sizeof(char) * 
+           (
+            strlen
+            (
+             "HTTP/1.0 503 Service Unavailable\r\nConnection: Close\r\n\r\nMobile C"
+            )+1
+           )
+          );
+        strcpy
+          (
+           buffer,
+           "HTTP/1.0 503 Service Unavailable\r\nConnection: Close\r\n\r\nMobile C"
+          );
+        send
+          (
+           connection->clientfd,
+           (void*)buffer,
+           sizeof(char)*(strlen(buffer)),
+           0
+          );
+        free(message_string);
+        free(buffer);
+        return ERR;
+      } else {
+        free(message_string);
+      }
+
+      if (n < SOCKET_INPUT_SIZE) {
+        buffer = strdup("HTTP/1.0 200 OK\r\nConnection: Close\r\n\r\nMobile C");
+        send(
+            connection->clientfd,
+            (void*)buffer,
+            sizeof(char)*strlen(buffer),
+            0 );
+#ifdef _WIN32
+        closesocket(connection->clientfd);
+#else
+        close(connection->clientfd);
+#endif
+        free(buffer);
+      }
       break;
     } else {
       message_size += n;
@@ -153,37 +202,7 @@ mtp_http_InitializeFromConnection
         break;
     }
   }
-  
-  if(mtp_http_Parse(http, message_string)) {
-    /* Reply with an HTTP error code */
-    buffer = malloc
-      (
-       sizeof(char) * 
-       (
-        strlen
-        (
-         "HTTP/1.0 503 Service Unavailable\r\nConnection: Close\r\n\r\nMobile C"
-        )+1
-       )
-      );
-    strcpy
-      (
-       buffer,
-       "HTTP/1.0 503 Service Unavailable\r\nConnection: Close\r\n\r\nMobile C"
-      );
-    send
-      (
-       connection->clientfd,
-       (void*)buffer,
-       sizeof(char)*(strlen(buffer)),
-       0
-      );
-    free(message_string);
-    return ERR;
-  } else {
-    free(message_string);
-    return 0;
-  }
+  return 0;
 }
 
 const char* http_GetExpression(const char* string, char** expr)
@@ -368,6 +387,7 @@ mtp_http_Parse(struct mtp_http_s* http, const char* string)
     HTTP_PARSE_EXPR( "User-Agent", user_agent)
     HTTP_PARSE_EXPR( "Cache-Control", cache_control)
     HTTP_PARSE_EXPR( "MIME-Version", mime_version)
+    HTTP_PARSE_EXPR( "Mime-Version", mime_version)
     {
       fprintf(stderr, "Warning: Unknown http name: %s. %s:%d\n",
           name, __FILE__, __LINE__);
@@ -387,26 +407,30 @@ mtp_http_Parse(struct mtp_http_s* http, const char* string)
   /* If the content type is multipart/mixed, then we need to
    * figure out how many parts there are. */
   if( 
+      http->content_type != NULL &&
       !strncmp(
         http->content_type,
-        "multipart-mixed",
-        strlen("multipart-mixed")
+        "multipart/mixed",
+        strlen("multipart/mixed")
         ) 
     ) 
   {
     tmp = strstr(http->content_type, "boundary=");
     tmp += strlen("boundary=.");
     tmp2 = strchr(tmp, '\"');
-    http->boundary = (char*)malloc(sizeof(char) * (tmp2 - tmp + 1));
+    http->boundary = (char*)malloc(sizeof(char) * (tmp2 - tmp + 3));
+    /* We must remember to include the leading '--' for the boundary */
+    http->boundary[0] = '-';
+    http->boundary[1] = '-';
     for (i = 0; tmp != tmp2; i++, tmp++) {
-      http->boundary[i] = *tmp;
+      http->boundary[i+2] = *tmp;
     }
-    http->boundary[i] = '\0';
+    http->boundary[i+2] = '\0';
 
     /* Count the number of message parts in the message */
     tmp = (char*)line;
     http->message_parts = 0;
-    while(tmp = strstr(tmp, http->boundary)) {
+    while((tmp = strstr(tmp, http->boundary))) {
       http->message_parts++;
       tmp++;
     }
@@ -426,7 +450,9 @@ mtp_http_Parse(struct mtp_http_s* http, const char* string)
          (strlen(line)+1)
         );
       strcpy((char*)http->content->data, line);
-      http->content->content_type = strdup(http->content_type);
+      if (http->content_type != NULL) {
+        http->content->content_type = strdup(http->content_type);
+      }
     }
   } else {
     http->content = (struct mtp_http_content_s*)malloc(
@@ -505,7 +531,8 @@ mtp_http_Parse(struct mtp_http_s* http, const char* string)
   }
   if (
       (http->http_performative == HTTP_POST) ||
-      (http->http_performative == HTTP_PUT)
+      (http->http_performative == HTTP_PUT) ||
+      (http->http_performative == HTTP_RESPONSE)
      )
     return 0;
   else
@@ -520,6 +547,7 @@ const char* http_ParseHeader(
   char* token;
   char* tmp = NULL;
   char* target;
+  int len;
 
   cur = string;
   cur = http_GetToken(cur, &token);
@@ -565,8 +593,24 @@ const char* http_ParseHeader(
     http->http_performative = HTTP_OPTIONS;
   } else if(!strcmp(token, "CONNECT")) {
     http->http_performative = HTTP_CONNECT;
-    /* FIXME */
+  } else if(!strncmp(token, "HTTP/", 5)) {
+    /* This is a response message */
+    http->http_performative = HTTP_RESPONSE;
+    /* We expect a status code */
+    free(token);
+    cur = http_GetToken(cur, &token);
+    sscanf(token, "%d", &http->response_code);
+    /* And now a status string */
+    len = strstr(cur, "\r\n") - 1;
+    http->response_string = malloc(
+        sizeof(char) * (len +1) );
+    strncpy(
+        http->response_string,
+        cur,
+        len );
+    http->response_string[len] = '\0';
   } else {
+    /* FIXME */
     /* Illegal performative */
     http->http_performative = HTTP_PERFORMATIVE_UNDEF;
   }
@@ -617,6 +661,11 @@ http_GetToken(const char* string, char** token)
   return cur+1;
 }
 
+int mtp_http_ParseResponse(struct mtp_http_s* http, const char* string)
+{
+
+}
+
 int 
 mtp_http_ComposeMessage(message_p message)
 {
@@ -672,69 +721,87 @@ mtp_http_CreateMessage(
   int i;
   int num;
   char buf[30];
+  char boundary[30];
   message_t* message;
-  dynstring_t* http_message;
-  http_message = dynstring_New();
-  dynstring_Append(http_message, "POST /");
-  dynstring_Append(http_message, mtp_http->target);
-  dynstring_Append(http_message, " HTTP/1.0\r\n");
-  dynstring_Append(http_message, "User-Agent: MobileC/");
-  dynstring_Append(http_message, PACKAGE_VERSION);
-  dynstring_Append(http_message, "\r\n");
-  dynstring_Append(http_message, "Host: ");
-  dynstring_Append(http_message, mtp_http->host);
-  dynstring_Append(http_message, "\r\n");
+  dynstring_t* http_header;
+  dynstring_t* http_body;
+  http_header = dynstring_New();
+  http_body = dynstring_New();
+  dynstring_Append(http_header, "POST /");
+  dynstring_Append(http_header, mtp_http->target);
+  dynstring_Append(http_header, " HTTP/1.1\r\n");
+  dynstring_Append(http_header, "User-Agent: MobileC/");
+  dynstring_Append(http_header, PACKAGE_VERSION);
+  dynstring_Append(http_header, "\r\n");
+  dynstring_Append(http_header, "Host: ");
+  dynstring_Append(http_header, mtp_http->host);
+  dynstring_Append(http_header, ":");
+  sprintf(buf, "%d", port);
+  dynstring_Append(http_header, buf);
+  dynstring_Append(http_header, "\r\n");
+  dynstring_Append(http_header, "Cache-Control: no-cache\r\n");
+  dynstring_Append(http_header, "Mime-Version: 1.0\r\n");
 
   /* The next part of the message will be different depending on 
    * how many message parts we have.  */
   if(mtp_http->message_parts == 1) {
-    dynstring_Append(http_message, "Content-Type: text/plain\r\n");
-    dynstring_Append(http_message, "Connection: Close");
-    dynstring_Append(http_message, "Content-Length: ");
+    dynstring_Append(http_header, "Content-Type: text/plain\r\n");
+    dynstring_Append(http_header, "Content-Length: ");
     sprintf(buf, "%d", strlen((char*)mtp_http->content[0].data));
-    dynstring_Append(http_message, buf);
-    dynstring_Append(http_message, "\r\n\r\n");
-    dynstring_Append(http_message, (char*)mtp_http->content[0].data);
+    dynstring_Append(http_header, buf);
+    dynstring_Append(http_header, "\r\n\r\n");
+    dynstring_Append(http_header, (char*)mtp_http->content[0].data);
   } else {
     /* We need to generate a random boundary. */
     srand(time(NULL));
-    strcpy(buf, "--");
+    strcpy(boundary, "--");
     for(i = 2; i < 26; i++) {
       num = rand() % 36;
       if(num < 10) {
-        buf[i] = (char)(48 + num);
+        boundary[i] = (char)(48 + num);
       } else {
-        buf[i] = (char)( (num-10)+65);
+        boundary[i] = (char)( (num-10)+65);
       }
     }
-    buf[i] = '\0';
-    /* FIXME: A randomly generated boundary should now be stored in buf. Should
+    boundary[i] = '\0';
+    /* FIXME: A randomly generated boundary should now be stored in boundary. Should
      * we check to see if there is a match within the body? The chances of collision are
      * infinitessimal. */
-    dynstring_Append(http_message, "Content-Type: multipart-mixed ; boundary=\"");
-    dynstring_Append(http_message, buf);
-    dynstring_Append(http_message, "\"\r\n\r\n");
-    dynstring_Append(http_message, "This is not part of the MIME multipart encoded message.\r\n");
+    dynstring_Append(http_body, "This is not part of the MIME multipart encoded message.\r\n");
     for(i = 0; i<mtp_http->message_parts; i++) {
-      dynstring_Append(http_message, buf);
-      dynstring_Append(http_message, "\nContent-Type: ");
-      dynstring_Append(http_message, mtp_http->content[i].content_type);
-      dynstring_Append(http_message, "\r\n\r\n");
-      dynstring_Append(http_message, (char*)mtp_http->content[i].data);
-      dynstring_Append(http_message, "\r\n\r\n");
+      dynstring_Append(http_body, boundary);
+      dynstring_Append(http_body, "\r\nContent-Type: ");
+      dynstring_Append(http_body, mtp_http->content[i].content_type);
+      dynstring_Append(http_body, "\r\n\r\n");
+      dynstring_Append(http_body, (char*)mtp_http->content[i].data);
+      dynstring_Append(http_body, "\r\n");
     }
+    /* Append one last boundary */
+    dynstring_Append(http_body, boundary);
+    dynstring_Append(http_body, "--");
+    dynstring_Append(http_body, "\r\n\r\n");
+
+    /* Set the content length in the header */
+    dynstring_Append(http_header, "Content-Length: ");
+    sprintf(buf, "%d", http_body->len-2 );
+    dynstring_Append(http_header, buf);
+    dynstring_Append(http_header, "\r\n");
+    dynstring_Append(http_header, "Content-Type: multipart/mixed ; boundary=\"");
+    dynstring_Append(http_header, boundary+2); /* Omit leading "--" */
+    dynstring_Append(http_header, "\"\r\n");
+
   }
-  /* Append one last boundary */
-  dynstring_Append(http_message, buf);
-  dynstring_Append(http_message, "\n");
+  dynstring_Append(http_header, "Connection: Close\r\n\r\n");
   message = message_New();
   message->to_address = (char*)malloc(
       sizeof(char) * (strlen(hostname)+15) );
   sprintf(message->to_address, "%s:%d", hostname, port);
   message->message_body = (char*) malloc( 
-      sizeof(char) * (http_message->len + 1));
-  strcpy(message->message_body, http_message->message);
-  dynstring_Destroy(http_message);
+      sizeof(char) * (http_header->len + http_body->len + 1));
+  strcpy(message->message_body, http_header->message);
+  strcat(message->message_body, http_body->message);
+  dynstring_Destroy(http_header);
+  dynstring_Destroy(http_body);
   message->isHTTP = 1;
   return message;
 }
