@@ -22,7 +22,7 @@ message_xml_compose__RequestEncryptionInit(mc_platform_p mc_platform)
   gaf_message_node = mxmlNewElement
     (
      node,
-     "GAF_MESSAGE"
+     "MOBILEC_MESSAGE"
     );
   message_node = mxmlNewElement
     (
@@ -61,13 +61,13 @@ message_xml_compose__EncryptionInitialize(mc_asm_p security_manager)
   node = mxmlLoadString
     (
      NULL,
-     "<?xml version=\"1.0\"?>\n<!DOCTYPE myMessage SYSTEM \"gafmessage.dtd\">",
+     "<?xml version=\"1.0\"?>\n<!DOCTYPE myMessage SYSTEM \"mobilec.dtd\">",
      MXML_NO_CALLBACK
     );
   gaf_message_node = mxmlNewElement
     (
      node,
-     "GAF_MESSAGE"
+     "MOBILEC_MESSAGE"
     );
   message_node = mxmlNewElement
     (
