@@ -5,17 +5,39 @@ using System.Runtime.InteropServices;
 
 /*!
  * \mainpage LibMC.NET
+ * 
+ * \author Douglas P. Stark<br>UC Davis and Sandia National Labs<br>dpstark\@sandia.gov
+ * 
  * \section intro_sec Introduction
  * Welcome to the LibMC.NET documentation. LibMC.NET is .NET class library
- * that wraps Mobile-C in a layer to make accessible to .NET applications.
+ * that wraps Mobile-C in a layer to make it accessible to .NET applications.
  * While the wrapper is continually under development, most of the features
- * of Mobile-C are currently available to the programmer. Please use the links
- * above to find out more.
- *
- * \section compiling Compiling LibMC.NET
- * Compiling LibMC.NET is straighforward but involves several steps. 
+ * of Mobile-C are currently available to the programmer. 
+ * \htmlonly 
+ * Please use the links above to find out more.
+ * \endhtmlonly
+ */
+
+/**
+ * \page installation Installing LibMC.NET
+ * Installing LibMC.NET is straighforward but involves several steps. 
  * 
- * \subsection downloading Downloading Mobile-C
+ * \section requirements Requirements
+ * In order to use LibMC.NET you will need the following:
+ * <ul>
+ * <li> Ch version 6.0.0 or greater from http://www.softintegration.com/.
+ * </li>
+ * <li> Embedded Ch version 6.0.0 or greater, also from http://www.softintegration.com/.
+ * </li>
+ * <li> Mobile-C 1.10.0 or greater. See Section \ref downloading for instructions 
+ * on how to obtain Mobile-C.
+ * </li>
+ * <li> Visual Studio 2005 or later. Express versions of Visual Studio can be found at 
+ * http://www.microsoft.com/express/.
+ * </li>
+ * </ul>
+ * 
+ * \section downloading Downloading Mobile-C
  * First, you must obtain a version of the Mobile-C source code. If you are reading
  * this, chances are you have already completed this step. If you have not already 
  * downloaded the source code, it can be done in one of three ways:
@@ -33,20 +55,20 @@ using System.Runtime.InteropServices;
  * <a href = "http://sourceforge.net/svn/?group_id=185466">this location.</a></li>
  * </ul>
  * 
- * \subsection build_mc Building the Mobile-C Libraries
+ * \section build_mc Building the Mobile-C Libraries
  * Once you have obtained the Mobile-C source, please see the Mobile-C User's Guide
  * for information on compiling Mobile-C under Windows. Currently, only the Visual 
  * Studio .NET 2005 project is supported for LibMC.NET. Section 2.3 of the User's 
  * Guide describes how to compile Mobile-C into a static library. For LibMC.NET,
- * at least one of two configurations are required: the "Debug_Dll" or "Release_DLL"
+ * at least one of two configurations are required: the "Debug_DLL" or "Release_DLL"
  * versions. To build either one, select the appropriate configuration (this replaces
  * step 3 in the User's Guide, Section 2.3.1) and build the solution (step 4). 
  * Alternatively, you may select "Batch Build" from the "Build" menu and build all
  * four possible configurations.
  * 
- * \subsection install_mc Install the Mobile-C Libraries
+ * \section install_mc Install the Mobile-C Libraries
  * After building the Mobile-C DLL files, the project will automatically copy the 
- * files to the system directory. By default, the files are copied to C:\Windows\System32\.
+ * files to the system directory. By default, the files are copied to C:/Windows/System32/.
  * If your system is configured differently or you wish to change the installation 
  * directory, right-click on the mc_lib_win32 project in the Solution Explorer and
  * select "Properties." In the mc_lib_win32 Property Pages treeview, select 
@@ -57,14 +79,20 @@ using System.Runtime.InteropServices;
  * You will also need to execute a "Rebuild" on the project to ensure the files are 
  * copied to the new location.
  * 
- * \subsection build_net Build LibMC.NET
+ * \section build_net Build LibMC.NET
  * Once you have built the Mobile-C DLL files, you can build LibMC.NET. Open the LibMC.NET
  * solution file located in the directory you installed or checked out Mobile-C to at 
  * src/win32/LibMC.NET/LibMC.Net.sln. From the "Build" menu, select "Rebuild Solution." 
  * You may want to build both the "Debug" and "Release" versions, or perform a batch build 
  * as described previously.
+ */
+
+/**
+ * \page started Getting Started
+ * LibMC.NET is very easy to use. The demo programs provided with the download are a good 
+ * place to start. Please see the Examples section for more information.
  * 
- * \subsection build_demo Build the Demo Programs
+ * \section build_demo Build the Demo Programs
  * The LibMC.NET demo program solution is located in the directory you installed or checked 
  * out Mobile-C to at demos/win32/LibMC.NET/LibMCDemos.sln. As before, select "Rebuild Solution"
  * from the "Build" menu. Note that the demo program solution contains the LibMC.NET project
@@ -76,30 +104,32 @@ using System.Runtime.InteropServices;
  * selecting "Debug" then "Start new instance." The demo programs have their own documentation
  * as well. See the README file in src/win32/LibMC.NET or demos/win32/LibMC.NET for information
  * on how to build the demo program documentation.
- * 
- * \section using Using LibMC.NET
+ */
+
+/**
+ * \page using Using LibMC.NET
  * This section explains how to use the LibMC.NET class library in your .NET project. Currently,
  * it only describes the process for using the library in a C# console or GUI application. Other
  * languages, such as VB and managed C++, will require similar actions.
  * 
- * \subsection project Create a Project
+ * \section project Create a Project
  * First, create the type of project you would like to use from the Visual Studio "Start Page" or
  * the "File" menu. Select the name and location of the project as you would any other project. 
  * Second, add a reference to the configuration of LibMC.NET you would like to use. For debugging
- * purposed, the "Debug" configuration is probably best. To add the reference, right click the 
+ * purposes, the "Debug" configuration is probably best. To add the reference, right click the 
  * "References" item in the Solution Explorer for the project you just created. Select the "Browse"
  * tab and navigate to the output directory of the LibMC.NET project. The directory is located at
- * src/win32/LibMC.NET/bin/<Configuration>/ in the Mobile-C source directory, where <Configuration>
+ * src/win32/LibMC.NET/bin/Configuration/ in the Mobile-C source directory, where Configuration
  * is either "Debug" or "Release." Select the DLL file and click "Ok." The References item in the 
  * Solution Explorer should now list "LibMC." Be sure to save the solution at this point.
  * 
- * \subsection code_use Using LibMC.NET Classes and Functions
+ * \section code_use Using LibMC.NET Classes and Functions
  * As with any other namespace, you must add the declaration "using LibMC;" to any file you want to
  * have access to the class libraries. Once you have added the using statement, you can declare
  * objects from the library as you normally would declare any other objects. See the example programs
  * for more details.
  * 
- * \subsection code_misc Other Options
+ * \section code_misc Other Options
  * You may want to enable one or more features in your project that can help you use LibMC.NET or
  * debug problems. If you add any XML files to your project, you probably will want to set their 
  * properties in the project to copy the files to the output directory. This is done by selecting
@@ -125,9 +155,122 @@ using System.Runtime.InteropServices;
  * a different location than the project. If you would like to be able to debug the Mobile-C library, 
  * you should select the "Enable unmanaged code debugging" check box. This will allow you to more 
  * easily see any errors that may occur in the unmanaged library, though hopefully none will.
+ */
+
+/**
+ * \page common_tasks Common Operations
+ * This section contains examples of commonly used operations for three main LibMC classes:
+ * <ul>
+ * <li>
+ * \subpage mcagency The mobile agent agency.
+ * </li>
+ * <li>
+ * \subpage mcagent Mobile agents.
+ * </li>
+ * <li>
+ * \subpage mcaclmessage Agent communication language messages.
+ * </li>
+ * </ul>
+ * 
+ * For complete programs and more detailed examples, see the Examples section.
+ */
+
+/**
+ * \page mcagency MCAgency
+ * Examples of commonly used MCAgency operations:
+ * 
+ * Declare an agency as a member of a class:
+ * \code
+ * public static MCAgency Agency = new MCAgency();
+ * \endcode
+ * 
+ * Set the agency's port:
+ * \code
+ * int temp = 5051;
+ * Agency.Port = temp;
+ * \endcode
  *
- * \section examples Example Projects
- * Please see the examples listed in the treeview to the left for more information.
+ * Start an agency:
+ * \code
+ * int temp;
+ * temp = Agency.Initialize();
+ * if (temp != 0)
+ *     Console.WriteLine("Initialize: " + temp.ToString());
+ * \endcode
+ * 
+ * Pause and resume an agency:
+ * \code
+ * Agency.HaltAgency();
+ * Agency.ResumeAgency();
+ * \endcode
+ * 
+ * Turn off the command prompt thread:
+ * \code
+ * temp = Agency.SetThreadOff(MCAgency.MC_ThreadIndex_e.MC_THREAD_CP);
+ * if (temp != 0)
+ *     Console.WriteLine("SetThreadOff: " + temp.ToString());
+ * \endcode
+ * 
+ * Load an agent into a local agency:
+ * \code
+ * String filename = "agent.xml";
+ * try
+ * {
+ *     Agency.LoadAgentMigrationMessageFile(filename);
+ * }
+ * catch (Exception ex)
+ * {
+ *     Console.WriteLine("Error loading file: " + ex.Message);
+ * }
+ * \endcode
+ * Ideally, the file name should be specified absolutely. 
+ * 
+ * Send an agent to a remote agency:
+ * \code
+ * String filename = "agent.xml";
+ * String ip = "192.168.23.93";
+ * int port = 5051;
+ * try
+ * {
+ *     Agency.SendAgentMigrationMessageFile(filename, ip, port);
+ * }
+ * catch (Exception ex)
+ * {
+ *     Console.WriteLine("Error sending file: " + ex.Message);
+ * }
+ * \endcode
+ * 
+ * Find an agent by name:
+ * \code
+ * MCAgent agent;
+ * try
+ * {
+ *     agent = Agency.FindAgentByName("persistent1");
+ * }
+ * catch (Exception e)
+ * {
+ *     Console.WriteLine("Exception: " + e.Message);
+ * }
+ * \endcode
+ *
+ * Wait for an agent to arrive:
+ * \code
+ * MCAgent agent;
+ * Agency.ResetSignal();
+ * try
+ * {
+ *     agent = Agency.WaitRetrieveAgent();
+ * }
+ * catch (Exception e)
+ * {
+ *     Console.WriteLine("Exception: " + e.Message);
+ * }
+ * \endcode
+ *
+ * Wait indefinitely while an agency runs:
+ * \code
+ * Agency.MainLoop();
+ * \endcode
  */
 
 /** \file MCAgency.cs
@@ -135,9 +278,8 @@ using System.Runtime.InteropServices;
  */
 
 /**
- * \example LibMCGui/Program.cs
- * \include LibMCGui/Form1.cs
- * \include LibMCGui/Form1.Designer.cs
+ * \example LibMCGui/Form1.cs
+ * \include LibMCGui/Program.cs
  * Basic Mobile-C Windows Forms demo program
  */
 
@@ -151,6 +293,21 @@ using System.Runtime.InteropServices;
  * Demonstrates miscellaneous Mobile-C functions.
  */
 
+/** 
+ * \example LibMCCppEx/LibMCCppEx.cpp
+ * \dontinclude LibMCCppEx/Form1.h
+ * \until #pragma region Windows Form Designer generated code
+ * \skip #pragma endregion
+ * \until };
+ * \line }
+ * Demonstrates using LibMC.NET from a VC++ program.
+ */
+
+/** 
+ * \example LibMCVbEx/Form1.vb
+ * Demonstrates using LibMC.NET from a VB program.
+ */
+
 /**
  * \brief      Namespace for the .NET wrapper for Mobile-C.
  * 
@@ -179,11 +336,22 @@ namespace LibMC
          */
         public enum MCAgencyState
         {
-            NoState = -1,       /*<! Default, uninitialized state */
-            Initialized = 0,    /*<! Agency initialized, but not started */
-            Running,            /*<! Agency is running */
-            Halted,             /*<! Agency has been stopped (can be resumed) */
-            Ended,              /*<! Agency is stopped (destroyed) */
+            NoState = -1,       /*!< Default, uninitialized state */
+            Initialized = 0,    /*!< Agency initialized, but not started */
+            Running,            /*!< Agency is running */
+            Halted,             /*!< Agency has been stopped (can be resumed) */
+            Ended,              /*!< Agency is stopped (destroyed) */
+        }
+
+        /**
+         * \brief   Ch shell type.
+         * 
+         * Used to set the shell type for the Ch interpretter.
+         */
+        public enum ChShellType
+        {
+            CH_REGULARCH = 0,   /*!< Default, regular shell */
+            CH_SAFECH = 1,      /*!< Safe shell */
         }
 
         private IntPtr agency_p = IntPtr.Zero;
@@ -314,11 +482,11 @@ namespace LibMC
          * 
          * \note        This function must be called before the agency is started.
          */
-        public int ChInitializeOptions(int shellType, String home)
+        public int ChInitializeOptions(ChShellType shellType, String home)
         {
             ChOptions_t options = new ChOptions_t();
             options.chhome = home;
-            options.shelltype = shellType;
+            options.shelltype = (int)shellType;
             return _MC_ChInitializeOptions(Agency, options);
         }
 
@@ -424,7 +592,7 @@ namespace LibMC
          * \param       status The enum that identifies the desired agent state.
          * \returns     The return value of the underlying MC_SetDefaultAgentStatus function.
          */
-        public int SetDefaultAgentStatus(MC_AgentStatus_e status)
+        public int SetDefaultAgentStatus(MCAgent.MC_AgentStatus_e status)
         {
             return _MC_SetDefaultAgentStatus(Agency, status);
         }
@@ -478,6 +646,20 @@ namespace LibMC
             return MCAgency._MC_SendAgentMigrationMessageFile(Agency, filename, hostname, port);
         }
 
+        /**
+         * \brief       Load an agent migration message.
+         * 
+         * Loads the specified XML file to this agency automatically. There is 
+         * no need to specify a port or agency location.
+         * 
+         * \param       filename The name of the file to send (fully qualified).
+         * \returns     The return value of the underlying MC_SendAgentMigrationMessageFile function.
+         */
+        public int LoadAgentMigrationMessageFile(String filename)
+        {
+            return MCAgency._MC_SendAgentMigrationMessageFile(Agency, filename, "localhost", this.Port);
+        }
+
         /**
          * \brief       Sends an agent migration message to an agency.
          * 
@@ -706,14 +888,33 @@ namespace LibMC
          * 
          * Really not sure.
          * 
-         * \returns     The return value of the underlying MC_CondBroadcast function.
+         * \returns     The return value of the underlying MC_SteerControl function.
+         * 
+         * \todo Test MC_SteerControl, MC_Steer.
          */
         public MC_SteerCommand_e SteerControl()
         {
             return _MC_SteerControl();
         }
 
+        /**
+         * \brief       Steering control function.
+         * 
+         * Really not sure.
+         * 
+         * \param       funcptr Pointer to the steering function
+         * \param       arg Argument to function
+         * \returns     The return value of the underlying _MC_Steer function.
+         * \note        This function does nothing but throw an exception right now.
+         * \bug         MC_Steer is not yet implemented.
+         * \todo        Implement MC_Steer
+         */
         /*public int _MC_Steer(IntPtr agency, int (*funcptr)(void* data), void *arg);*/
+        public int Steer(IntPtr funcptr, IntPtr arg)
+        {
+            //return _MC_Steer(Agency, IntPtr funcptr, IntPtr arg);
+            throw new Exception("Steer() is not yet implemented!");
+        }
 
         /*
          * Services
@@ -731,13 +932,37 @@ namespace LibMC
          * \param       serviceNames An array of service names.
          * \param       numServices The number of services provided.
          * \returns     The return value of the underlying MC_RegisterService function.
+         * 
+         * \todo        Test MC_RegisterService and MC_SearchForService.
          */
         public int RegisterService(MCAgent agent, int agentID, String agentName, String[] serviceNames, int numServices)
         {
             return _MC_RegisterService(Agency, agent.Agent, agentID, agentName, serviceNames, numServices);
         }
 
-        /*public int _MC_SearchForService(IntPtr agency, String searchString, char*** agentNames, char*** serviceNames, int** agentIDs, int* numResults);*/
+        /**
+         * \brief       Searches for services in the agency.
+         * 
+         * Searches for services provided by agents with the agency. Not really useful
+         * in binary space.
+         * 
+         * \param       searchString The agent providing the services.
+         * \param       agentNames The agent ID number.
+         * \param       serviceNames The agent name.
+         * \param       agentIDs An array of service names.
+         * \param       numResults The number of services provided.
+         * \returns     The return value of the underlying MC_SearchForService function.
+         * 
+         * \note        This function does nothing but throw an exception right now. 
+         * \bug         MC_SearchForService is not yet implemented.
+         * \todo        Implement SearchForService
+         */
+        /*private static extern int _MC_SearchForService(IntPtr agency, String searchString, char*** agentNames, char*** serviceNames, int** agentIDs, int* numResults);*/
+        public int SearchForService(String searchString, IntPtr agentNames, IntPtr serviceNames, IntPtr agentIDs, IntPtr numResults)
+        {
+            //return _MC_SearchForService(IntPtr agency, String searchString, IntPtr agentNames, IntPtr serviceNames, IntPtr agentIDs, IntPtr numResults);
+            throw new Exception("SearchForService() is not yet implemented!");
+        }
 
         /*
          * Agent functions
@@ -810,5 +1035,18 @@ namespace LibMC
         {
             return MCAgency._MC_AclSend(Agency, acl_message.AclMsg);
         }
+
+        /**
+         * \brief       Makes the agency wait indefinitely.
+         * 
+         * Makes the agency wait indefinitely until it receives a "quit" command
+         * or is otherwise terminated.
+         * 
+         * \returns     The return value of the underlying MC_MainLoop function.
+         */
+        public int MainLoop()
+        {
+            return _MC_MainLoop(Agency);
+        }
     }
 }
