@@ -9,68 +9,6 @@ using System.Runtime.InteropServices;
 
 namespace LibMC
 {
-    /**
-     * \brief   Enum for describing the type of an agent.
-     * 
-     * \note    This enum is pulled directly from the Mobile-C library.
-     */
-    public enum MC_AgentType_e
-    {
-        MC_NONE = -1,           /**< Default value to describe unininitialized agent. */
-        MC_REMOTE_AGENT = 0,    /**< A remote agent. */
-        MC_LOCAL_AGENT,         /**< A local agent. */
-        MC_RETURN_AGENT         /**< A returning agent. */
-    };
-
-    /**
-     * \brief   Enum for describing the status of an agent.
-     * 
-     * \note    This enum is pulled directly from the Mobile-C library.
-     */
-    public enum MC_AgentStatus_e
-    {
-        MC_NO_STATUS = -1, /*!< Default value for uninitialized agent */
-        MC_WAIT_CH = 0,    /*!< Waiting to be started */
-        MC_WAIT_MESSGSEND, /*!< Finished, waiting to migrate */
-        MC_AGENT_ACTIVE,   /*!< Running */
-        MC_AGENT_NEUTRAL,  /*!< Not running, but do not flush */
-        MC_AGENT_SUSPENDED,/*!< Unused */
-        MC_WAIT_FINISHED   /*!< Finished, waiting to be flushed */
-    };
-
-    /**
-     * \brief   Enum for describing the type of an ACL message.
-     * 
-     * \note    This enum is pulled directly from the Mobile-C library.
-     */
-    public enum MC_FipaPerformative_e
-    {
-        FIPA_ERROR = -1,            /*!< Fipa performative enum value */
-        FIPA_ZERO,                  /*!< Fipa performative enum value */
-        FIPA_ACCEPT_PROPOSAL,       /*!< Fipa performative enum value */
-        FIPA_AGREE,                 /*!< Fipa performative enum value */
-        FIPA_CANCEL,                /*!< Fipa performative enum value */
-        FIPA_CALL_FOR_PROPOSAL,     /*!< Fipa performative enum value */
-        FIPA_CONFIRM,               /*!< Fipa performative enum value */
-        FIPA_DISCONFIRM,            /*!< Fipa performative enum value */
-        FIPA_FAILURE,               /*!< Fipa performative enum value */
-        FIPA_INFORM,                /*!< Fipa performative enum value */
-        FIPA_INFORM_IF,             /*!< Fipa performative enum value */
-        FIPA_INFORM_REF,            /*!< Fipa performative enum value */
-        FIPA_NOT_UNDERSTOOD,        /*!< Fipa performative enum value */
-        FIPA_PROPOGATE,             /*!< Fipa performative enum value */
-        FIPA_PROPOSE,               /*!< Fipa performative enum value */
-        FIPA_PROXY,                 /*!< Fipa performative enum value */
-        FIPA_QUERY_IF,              /*!< Fipa performative enum value */
-        FIPA_QUERY_REF,             /*!< Fipa performative enum value */
-        FIPA_REFUSE,                /*!< Fipa performative enum value */
-        FIPA_REJECT_PROPOSAL,       /*!< Fipa performative enum value */
-        FIPA_REQUEST,               /*!< Fipa performative enum value */
-        FIPA_REQUEST_WHEN,          /*!< Fipa performative enum value */
-        FIPA_REQUEST_WHENEVER,      /*!< Fipa performative enum value */
-        FIPA_SUBSCRIBE              /*!< Fipa performative enum value */
-    };
-
     public partial class MCAgency
     {
         //private const String mcdll = "C:\\mobilec\\src\\win32\\vcnet2005\\dll debug\\libmcd.dll";
@@ -81,9 +19,6 @@ namespace LibMC
             "libmc.dll";
 #endif
 
-        /**
-         * \brief   TACOS!
-         */
         [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
         internal struct MCAgency_t
         {
@@ -143,7 +78,7 @@ namespace LibMC
          */
         public enum MC_ThreadIndex_e
         {
-            MC_THREAD_DF = 0,   /*!< Directory Facilitator */
+            MC_THREAD_DF = 0, /*!< Directory Facilitator */
             MC_THREAD_AMS,    /*!< Agent Managment system */
             MC_THREAD_ACC,    /*!< Agency communications */
             MC_THREAD_CP,     /*!< Command Prompt */
@@ -208,11 +143,13 @@ namespace LibMC
         [DllImport(mcdll, EntryPoint = "MC_ResumeAgency")]
         private static extern int _MC_ResumeAgency(IntPtr agency);
         [DllImport(mcdll, EntryPoint = "MC_SetDefaultAgentStatus")]
-        private static extern int _MC_SetDefaultAgentStatus(IntPtr agency, MC_AgentStatus_e status);
+        private static extern int _MC_SetDefaultAgentStatus(IntPtr agency, MCAgent.MC_AgentStatus_e status);
         [DllImport(mcdll, EntryPoint = "MC_WaitRetrieveAgent")]
         private static extern IntPtr _MC_WaitRetrieveAgent(IntPtr agency);
         [DllImport(mcdll, EntryPoint = "MC_WaitAgent")]
         private static extern int _MC_WaitAgent(IntPtr agency);
+        [DllImport(mcdll, EntryPoint = "MC_ MainLoop")]
+        private static extern int _MC_MainLoop(IntPtr agency);
 
         // Migration functions
         [DllImport(mcdll, EntryPoint = "MC_SendAgentMigrationMessageFile")]
@@ -252,15 +189,17 @@ namespace LibMC
 
         //Steering functions
         [DllImport(mcdll, EntryPoint = "MC_SteerControl")]
-        public static extern MC_SteerCommand_e _MC_SteerControl();
-        /*[DllImport(dllfile, EntryPoint = "MC_Steer")]
-        private static extern int _MC_Steer(IntPtr agency, int (*funcptr)(void* data), void *arg);*/
+        private static extern MC_SteerCommand_e _MC_SteerControl();
+        [DllImport(mcdll, EntryPoint = "MC_Steer")]
+        private static extern int _MC_Steer(IntPtr agency, IntPtr funcptr, IntPtr arg);
+        /*private static extern int _MC_Steer(IntPtr agency, int (*funcptr)(void* data), void *arg);*/
 
         // Services
         [DllImport(mcdll, EntryPoint = "MC_RegisterService")]
         private static extern int _MC_RegisterService(IntPtr agency, IntPtr agent, int agentID, String agentName, String[] serviceNames, int numServices);
-        /*[DllImport(dllfile, EntryPoint = "MC_SearchForService")]
-        private static extern int _MC_SearchForService(IntPtr agency, String searchString, char*** agentNames, char*** serviceNames, int** agentIDs, int* numResults);*/
+        [DllImport(mcdll, EntryPoint = "MC_SearchForService")]
+        private static extern int _MC_SearchForService(IntPtr agency, String searchString, IntPtr agentNames, IntPtr serviceNames, IntPtr agentIDs, IntPtr numResults);
+        /*private static extern int _MC_SearchForService(IntPtr agency, String searchString, char*** agentNames, char*** serviceNames, int** agentIDs, int* numResults);*/
 
         // Agent-related agency functions
         [DllImport(mcdll, EntryPoint = "MC_AddAgent")]
@@ -282,9 +221,9 @@ namespace LibMC
         [DllImport(mcdll, EntryPoint = "MC_GetAgentNumTasks")]
         internal static extern int _MC_GetAgentNumTasks(IntPtr agent);
         [DllImport(mcdll, EntryPoint = "MC_GetAgentStatus")]
-        internal static extern MC_AgentStatus_e _MC_GetAgentStatus(IntPtr agent);
+        internal static extern MCAgent.MC_AgentStatus_e _MC_GetAgentStatus(IntPtr agent);
         [DllImport(mcdll, EntryPoint = "MC_GetAgentType")]
-        internal static extern MC_AgentType_e _MC_GetAgentType(IntPtr agent);
+        internal static extern MCAgent.MC_AgentType_e _MC_GetAgentType(IntPtr agent);
         [DllImport(mcdll, EntryPoint = "MC_GetAgentXMLString")]
         internal static extern String _MC_GetAgentXMLString(IntPtr agent);
         [DllImport(mcdll, EntryPoint = "MC_PrintAgentCode")]
@@ -292,15 +231,17 @@ namespace LibMC
         [DllImport(mcdll, EntryPoint = "MC_RetrieveAgentCode")]
         internal static extern String _MC_RetrieveAgentCode(IntPtr agent);
         [DllImport(mcdll, EntryPoint = "MC_SetAgentStatus")]
-        internal static extern int _MC_SetAgentStatus(IntPtr agent, MC_AgentStatus_e status);
+        internal static extern int _MC_SetAgentStatus(IntPtr agent, MCAgent.MC_AgentStatus_e status);
         [DllImport(mcdll, EntryPoint = "MC_TerminateAgent")]
         internal static extern int _MC_TerminateAgent(IntPtr agent);
-        /*[DllImport(dllfile, EntryPoint = "MC_CallAgentFunc")]
-        internal static extern int _MC_CallAgentFunc(IntPtr agent, String funcName, void* returnVal, void* varg);*/
-        /*[DllImport(dllfile, EntryPoint = "MC_GetAgentExecEngine")]
-        internal static extern void* _MC_GetAgentExecEngine(IntPtr agent);*/
-        /*[DllImport(dllfile, EntryPoint = "MC_GetAgentReturnData")]
-        internal static extern int _MC_GetAgentReturnData(IntPtr agent, int task_num, void **data, int *dim, int **extent);*/
+
+        [DllImport(mcdll, EntryPoint = "MC_CallAgentFunc")]
+        internal static extern int _MC_CallAgentFunc(IntPtr agent, String funcName, IntPtr returnVal, IntPtr varg);
+        [DllImport(mcdll, EntryPoint = "MC_GetAgentExecEngine")]
+        internal static extern IntPtr _MC_GetAgentExecEngine(IntPtr agent);
+        [DllImport(mcdll, EntryPoint = "MC_GetAgentReturnData")]
+        internal static extern int _MC_GetAgentReturnData(IntPtr agent, int task_num, IntPtr data, IntPtr dim, IntPtr extent);
+        /*internal static extern int _MC_GetAgentReturnData(IntPtr agent, int task_num, void **data, int *dim, int **extent);*/
 
         // If the return is an Intptr, it's really a fipa_acl_message_s pointer.
         // If an argument is IntPtr, it's a pointer to a fipa_acl_message_s
@@ -320,7 +261,7 @@ namespace LibMC
 
         /* ACL Helper Functions Here */
         [DllImport(mcdll, EntryPoint = "MC_AclSetPerformative")]
-        internal static extern int _MC_AclSetPerformative(IntPtr acl, MC_FipaPerformative_e performative);
+        internal static extern int _MC_AclSetPerformative(IntPtr acl, MCAclMessage.MC_FipaPerformative_e performative);
         [DllImport(mcdll, EntryPoint = "MC_AclSetSender")]
         internal static extern int _MC_AclSetSender(IntPtr acl, String name, String address);
         [DllImport(mcdll, EntryPoint = "MC_AclAddReceiver")]
