@@ -41,7 +41,7 @@ agent_xml_compose(agent_p agent)
   node = mxmlLoadString
     (
      NULL,
-     "<?xml version=\"1.0\"?>\n<!DOCTYPE myMessage SYSTEM \"gafmessage.dtd\">",
+     "<?xml version=\"1.0\"?>\n<!DOCTYPE myMessage SYSTEM \"mobilec.dtd\">",
      MXML_NO_CALLBACK
     );
   mxmlAdd
@@ -61,7 +61,7 @@ agent_xml_compose__gaf_message(agent_p agent)
   node = mxmlNewElement
     (
      NULL,
-     "GAF_MESSAGE"
+     "MOBILEC_MESSAGE"
     );
   mxmlAdd
     (
@@ -185,7 +185,7 @@ agent_xml_compose__agent_data(agent_p agent)
   }
 
   /* Add the 'task' node */
-  tmp_node = agent_xml_compose__task(agent);
+  tmp_node = agent_xml_compose__tasks(agent);
   if (tmp_node != NULL) {
     mxmlAdd(
         node,
@@ -247,16 +247,17 @@ agent_xml_compose__home(agent_p agent)
 }
 
   mxml_node_t*
-agent_xml_compose__task(agent_p agent)
+agent_xml_compose__tasks(agent_p agent)
 {
   char buf[30];
+  int i;
   mxml_node_t* node;
   mxml_node_t* tmp_node;
-  int i;
-  node = mxmlNewElement(
+
+  node=mxmlNewElement(
       NULL,
-      "TASK"
-      );
+      "TASKS" );
+
   sprintf(buf, "%d", agent->datastate->number_of_tasks);
   mxmlElementSetAttr(
       node,
@@ -273,7 +274,7 @@ agent_xml_compose__task(agent_p agent)
       );
 
   for (i = 0; i < agent->datastate->number_of_tasks; i++) {
-    tmp_node = agent_xml_compose__data(agent, i);
+    tmp_node = agent_xml_compose__task(agent, i);
     if (tmp_node != NULL) {
       mxmlAdd(
           node,
@@ -285,6 +286,7 @@ agent_xml_compose__task(agent_p agent)
       return NULL;
     }
   }
+
   i=0;
   tmp_node = agent_xml_compose__agent_code(agent, i);
   while (tmp_node != NULL) {
@@ -302,31 +304,23 @@ agent_xml_compose__task(agent_p agent)
 }
 
   mxml_node_t*
-agent_xml_compose__data(agent_p agent, int index)
+agent_xml_compose__task(agent_p agent, int index)
 {
   char buf[30];
   mxml_node_t* node;
   mxml_node_t* tmp_node;
+  interpreter_variable_data_t* tmp_interp_var;
   node = mxmlNewElement(
       NULL,
-      "DATA"
+      "TASK"
       );
-  /* Set up return variable name attribute */
-  if ( agent->datastate->tasks[index]->var_name != NULL ) {
-    mxmlElementSetAttr(
-        node,
-        "name",
-        agent->datastate->tasks[index]->var_name
-        );
-  }
 
-  /* Set up task completion attribute */
-  sprintf(buf, "%d", agent->datastate->tasks[index]->task_completed);
+  buf[0] = '\0';
+  sprintf(buf, "%d", index);
   mxmlElementSetAttr(
       node,
-      "complete",
-      buf
-      );
+      "num",
+      buf );
 
   /* Set up server name attribute */
   mxmlElementSetAttr(
@@ -335,14 +329,13 @@ agent_xml_compose__data(agent_p agent, int index)
       agent->datastate->tasks[index]->server_name
       );
 
-  /* Set up persistent flag, only if it _is_ persistent */
-  if (agent->datastate->persistent || agent->datastate->tasks[index]->persistent) {
-    mxmlElementSetAttr(
-        node,
-        "persistent",
-        "1"
-        );
-  }
+  /* Set up task completion attribute */
+  sprintf(buf, "%d", agent->datastate->tasks[index]->task_completed);
+  mxmlElementSetAttr(
+      node,
+      "complete",
+      buf
+      );
 
   /* Set up the code id */
   if (agent->datastate->tasks[index]->code_id) {
@@ -353,73 +346,148 @@ agent_xml_compose__data(agent_p agent, int index)
         );
   }
 
-  /* If the task is not complete, we may return here. Otherwise, we have
-   * to deal with the return data. */
-  if (!agent->datastate->tasks[index]->task_completed) {
-    return node;
+  /* Set up persistent flag, only if it _is_ persistent */
+  if (agent->datastate->persistent || agent->datastate->tasks[index]->persistent) {
+    mxmlElementSetAttr(
+        node,
+        "persistent",
+        "1"
+        );
   }
 
-  /* *****************************
-   * Set up agent return attributes here
-   * ***************************** */
-  if (agent->datastate->tasks[index]->var_name != NULL) {
-    if (
-        strcmp(
+  /* Add 'DATA' nodes */
+  /* First, add the return data */
+  if(agent->datastate->tasks[index]->var_name != NULL) {
+    mxmlElementSetAttr(
+        node,
+        "return",
+        agent->datastate->tasks[index]->var_name );
+
+    if(
+        strcmp( 
           "no-return",
           agent->datastate->tasks[index]->var_name
           )
-       )
+        &&
+        agent->datastate->tasks[index]->task_completed
+      )
     {
-      if (agent->datastate->tasks[index]->agent_return_data != NULL)
-      {
-        /* Set up the 'dim' attribute */
-        sprintf(
-            buf, 
-            "%d", 
-            agent->datastate->tasks[index]->agent_return_data->array_dim
-            );
-        mxmlElementSetAttr(
-            node,
-            "dim",
-            buf
-            );
 
-        /* set up the 'type' attribute */
-        CH_DATATYPE_STRING(
-            agent->datastate->tasks[index]->agent_return_data->data_type,
-            buf
-            );
-        mxmlElementSetAttr(
-            node,
-            "type",
-            buf
-            );
+      if (agent->datastate->tasks[index]->agent_return_data != NULL) {
+        tmp_node = agent_xml_compose__data(
+            agent, 
+            index, 
+            agent->datastate->tasks[index]->agent_return_data);
+      } else {tmp_node = NULL;}
 
-        if (agent->datastate->tasks[index]->agent_return_data->array_dim == 0)
-        {
-          CH_DATATYPE_VALUE_STRING
-            (
-             agent->datastate->tasks[index]->agent_return_data->data_type,
-             buf,
-             agent->datastate->tasks[index]->agent_return_data->return_data
+      if(tmp_node == NULL) {
+        fprintf(stderr, "Compose error. %s:%d\n", __FILE__, __LINE__);
+        return NULL;
+      }
+      mxmlAdd(
+          node,
+          MXML_ADD_AFTER,
+          NULL,
+          tmp_node );
+    }
+  }
+
+  /* Now, add all the variables that the agent wants to save */
+  while
+    (
+     ( 
+      tmp_interp_var = agent_variable_list_Pop(
+        agent->datastate->tasks[index]->agent_variable_list )
+     ) != NULL 
+    )
+  {
+    tmp_node = agent_xml_compose__data(
+        agent,
+        index,
+        tmp_interp_var);
+    free(tmp_interp_var);
+    if(tmp_node == NULL) {
+      fprintf(stderr, "Compose error. %s:%d\n", __FILE__, __LINE__);
+      return NULL;
+    }
+    mxmlAdd(
+        node,
+        MXML_ADD_AFTER,
+        NULL,
+        tmp_node );
+  }
+
+  return node;
+}
+
+  mxml_node_t*
+agent_xml_compose__data(agent_p agent, int index, interpreter_variable_data_t* interp_variable)
+{
+  char buf[30];
+  mxml_node_t* node;
+  mxml_node_t* tmp_node;
+
+  if (interp_variable == NULL) { return NULL; }
+
+  node = mxmlNewElement(
+      NULL,
+      "DATA"
+      );
+  /* Set up return variable name attribute */
+  mxmlElementSetAttr(
+      node,
+      "name",
+      interp_variable->name
+      );
+
+  if (interp_variable != NULL)
+  {
+    /* Set up the 'dim' attribute */
+    sprintf(
+        buf, 
+        "%d", 
+        interp_variable->array_dim
+        );
+    mxmlElementSetAttr(
+        node,
+        "dim",
+        buf
+        );
+
+    /* set up the 'type' attribute */
+    CH_DATATYPE_STRING(
+        interp_variable->data_type,
+        buf
+        );
+    mxmlElementSetAttr(
+        node,
+        "type",
+        buf
+        );
+
+    if (interp_variable->array_dim == 0)
+    {
+      CH_DATATYPE_VALUE_STRING
+        (
+         interp_variable->data_type,
+         buf,
+         interp_variable->data
+        );
+      mxmlElementSetAttr(
+          node,
+          "value",
+          buf
+          );
+    } else {
+      /* Set up row nodes */
+      tmp_node = agent_xml_compose__row(interp_variable, index);
+      if (tmp_node != NULL) {
+        mxmlAdd(
+            node,
+            MXML_ADD_AFTER,
+            NULL,
+            tmp_node
             );
-          mxmlElementSetAttr(
-              node,
-              "return_value",
-              buf
-              );
-        } else {
-          /* Set up row nodes */
-          tmp_node = agent_xml_compose__row(agent, index);
-          if (tmp_node != NULL) {
-            mxmlAdd(
-                node,
-                MXML_ADD_AFTER,
-                NULL,
-                tmp_node
-                );
-          }
-        }
       }
     }
   }
@@ -455,21 +523,21 @@ agent_xml_compose__agent_code(agent_p agent, int index)
 }
 
   mxml_node_t*
-agent_xml_compose__row(agent_p agent, int index)
+agent_xml_compose__row(interpreter_variable_data_t* interp_variable, int index)
 {
   mxml_node_t* node;
 
-  if (agent->datastate->tasks[index]->agent_return_data == NULL) {
+  if (interp_variable == NULL) {
     return NULL;
   }
 
   node = agent_xml_compose__create_row_nodes
     (
-     agent->datastate->tasks[index]->agent_return_data->return_data,
+     interp_variable->data,
      0,
-     agent->datastate->tasks[index]->agent_return_data->array_extent,
-     agent->datastate->tasks[index]->agent_return_data->data_type,
-     agent->datastate->tasks[index]->agent_return_data->array_dim,
+     interp_variable->array_extent,
+     interp_variable->data_type,
+     interp_variable->array_dim,
      0
     );
   return node;
