@@ -31,6 +31,7 @@
 
 #include <mxml.h>
 #include <string.h>
+#include <stdlib.h>
 #define _XOPEN_SOURCE 600
 #include <stdlib.h>
 #ifndef _WIN32
@@ -38,10 +39,11 @@
 #else
 #include "winconfig.h"
 #endif
-#include "include/agent_return_data.h"
+#include "include/interpreter_variable_data.h"
 #include "include/message.h"
 #include "include/xml_parser.h"
 #include "include/xml_helper.h"
+
 /* *** */
 /* agent_xml_parse */
 error_code_t agent_xml_parse(agent_p agent)
@@ -128,10 +130,10 @@ agent_xml_parse__agent_data
 
   xml_parser->node = xml_get_child(
       agent_data_node,
-      "TASK",
+      "TASKS",
       1
       );
-  if ( (err_code = agent_xml_parse__task(agent, xml_parser)) ) {
+  if ( (err_code = agent_xml_parse__tasks(agent, xml_parser)) ) {
     return err_code;
   }
   return MC_SUCCESS;
@@ -217,21 +219,22 @@ agent_xml_parse__home(agent_p agent, xml_parser_p xml_parser)
 }
 
 /* *** */
-/* agent_xml_parse__task */
+/* agent_xml_parse__tasks */
 error_code_t
-agent_xml_parse__task(agent_p agent, xml_parser_p xml_parser)
+agent_xml_parse__tasks(agent_p agent, xml_parser_p xml_parser)
 {
-  const char* attribute;
-  const mxml_node_t* task_node;
   int i;
-  int task_num;
-  int code_num = 0;
-  error_code_t err_code = MC_SUCCESS;
-  CHECK_NULL(xml_parser->node, return MC_ERR_PARSE;);
-  task_node = xml_parser->node;
+  int code_num=0;
+  int err_code;
+  const char* attribute;
+  const mxml_node_t* tasks_node;
+  char buf[20];
+
+  tasks_node = xml_parser->node;
+
   /* parse the 'task' attribute */
   attribute = mxmlElementGetAttr(
-      (mxml_node_t*)task_node,
+      (mxml_node_t*)tasks_node,
       "task"
       );
   if (attribute == NULL) {
@@ -243,17 +246,9 @@ agent_xml_parse__task(agent_p agent, xml_parser_p xml_parser)
       sizeof(agent_task_p) * agent->datastate->number_of_tasks
       );
 
-  /* Allocate each task */
-  for(i = 0; i<agent->datastate->number_of_tasks; i++) {
-    agent->datastate->tasks[i] = (agent_task_p)malloc
-      (
-       sizeof(agent_task_t)
-      );
-  }
-
   /* parse the 'num' attribute */
   attribute = mxmlElementGetAttr(
-      (mxml_node_t*)task_node,
+      (mxml_node_t*)tasks_node,
       "num"
       );
   if (attribute == NULL) {
@@ -262,22 +257,28 @@ agent_xml_parse__task(agent_p agent, xml_parser_p xml_parser)
     agent->datastate->task_progress = atoi((char*)attribute);
   }
 
-  /* Parse the multiple DATA nodes */
-  for (i = 0; i < agent->datastate->number_of_tasks; i++) {
-    if (i == 0) {
-      xml_parser->node = xml_get_child(
-          task_node,
-          "DATA",
-          1
-          );
-    }
-    if(i != 0) {
-      xml_parser->node = xml_get_next_element(xml_parser->node);
-    }
-    if ((err_code = agent_xml_parse__data(agent, xml_parser, i)))
-    {
-      return err_code;
+  /* Allocate each task */
+  for(i = 0; i<agent->datastate->number_of_tasks; i++) {
+    agent->datastate->tasks[i] = agent_task_New();
+  }
+
+  /* Parse each task */
+  for(i = 0; i < agent->datastate->number_of_tasks; i++) {
+    sprintf(buf, "%d", i);
+    xml_parser->node = mxmlFindElement(
+        tasks_node,
+        tasks_node,
+        "TASK",
+        "num",
+        buf,
+        MXML_DESCEND_FIRST );
+    if(xml_parser->node == NULL) {
+      fprintf(stderr,
+          "ERROR: Could not find task num %d! %s:%d\n",
+          i, __FILE__, __LINE__);
+      return MC_ERR_PARSE;
     }
+    agent_xml_parse__task(agent, xml_parser, i);
   }
 
   /* Need to get all of the agent codes. Even though we may execute only
@@ -285,21 +286,22 @@ agent_xml_parse__task(agent_p agent, xml_parser_p xml_parser)
    * to execute, so we need them all. */
   xml_parser->node = mxmlFindElement
       (
-       task_node,
-       task_node,
+       tasks_node,
+       tasks_node,
        "AGENT_CODE",
        NULL,
        NULL,
        MXML_DESCEND
       );
 
+
   /* First we count the number of code blocks */
   while(xml_parser->node != NULL) {
     code_num++;
     xml_parser->node = mxmlFindElement
       (
        xml_parser->node,
-       task_node,
+       tasks_node,
        "AGENT_CODE",
        NULL,
        NULL,
@@ -323,8 +325,8 @@ agent_xml_parse__task(agent_p agent, xml_parser_p xml_parser)
   /* Go through all code again and parse */
   xml_parser->node = mxmlFindElement
       (
-       task_node,
-       task_node,
+       tasks_node,
+       tasks_node,
        "AGENT_CODE",
        NULL,
        NULL,
@@ -337,7 +339,7 @@ agent_xml_parse__task(agent_p agent, xml_parser_p xml_parser)
     xml_parser->node = mxmlFindElement
       (
        xml_parser->node,
-       task_node,
+       tasks_node,
        "AGENT_CODE",
        NULL,
        NULL,
@@ -351,6 +353,98 @@ agent_xml_parse__task(agent_p agent, xml_parser_p xml_parser)
     return MC_ERR_PARSE;
   }
 
+  return 0;
+}
+
+/* *** */
+/* agent_xml_parse__task */
+error_code_t
+agent_xml_parse__task(agent_p agent, xml_parser_p xml_parser, int index)
+{
+  const char* attribute;
+  const mxml_node_t* task_node;
+  error_code_t err_code = MC_SUCCESS;
+  CHECK_NULL(xml_parser->node, return MC_ERR_PARSE;);
+  task_node = xml_parser->node;
+
+  /* Parse the multiple DATA nodes */
+  xml_parser->node = mxmlFindElement(
+      task_node,
+      task_node,
+      "DATA",
+      NULL,
+      NULL,
+      MXML_DESCEND_FIRST);
+  while(xml_parser->node != NULL) {
+    /* There may be no DATA nodes */
+    if ((err_code = agent_xml_parse__data(agent, xml_parser, index)))
+    {
+      return err_code;
+    }
+    xml_parser->node = mxmlFindElement(
+        xml_parser->node,
+        task_node,
+        "DATA",
+        NULL,
+        NULL,
+        MXML_NO_DESCEND );
+  }
+
+  /* 'code_id' */
+  attribute = mxmlElementGetAttr(
+      (mxml_node_t*)task_node,
+      "code_id");
+  if (attribute != NULL) {
+    agent->datastate->tasks[index]->code_id = malloc
+      (
+       sizeof(char) * 
+       (strlen(attribute) + 1)
+      );
+    strcpy(agent->datastate->tasks[index]->code_id, attribute);
+  } else {
+    agent->datastate->tasks[index]->code_id = NULL;
+  }
+
+  /* 'num' - The number of this task (indexed from 0) */
+  attribute = mxmlElementGetAttr(
+      (mxml_node_t*)task_node,
+      "num"
+      );
+  CHECK_NULL(attribute, return MC_ERR_PARSE;);
+
+  /* 'complete' - Is this task complete?*/
+  attribute = mxmlElementGetAttr(
+      (mxml_node_t*)task_node,
+      "complete"
+      );
+  CHECK_NULL(attribute, return MC_ERR_PARSE;);
+  agent->datastate->tasks[index]->task_completed = 
+    atoi((char*)attribute);
+  
+  /* 'server' - The server this task should be performed on */
+  attribute = mxmlElementGetAttr(
+      (mxml_node_t*)task_node,
+      "server"
+      );
+  CHECK_NULL(attribute, return MC_ERR_PARSE;);
+  agent->datastate->tasks[index]->server_name = 
+    (char*)malloc(sizeof(char) * (strlen(attribute)+1) );
+  strcpy(
+      agent->datastate->tasks[index]->server_name,
+      attribute
+      );
+
+  /* 'return' - The name of the return variable, if there is one */
+  attribute = mxmlElementGetAttr(
+      (mxml_node_t*)task_node,
+      "return" );
+  if (attribute == NULL) {
+    agent->datastate->tasks[index]->var_name = strdup("no-return");
+  } else {
+    agent->datastate->tasks[index]->var_name = strdup(attribute);
+  }
+  CHECK_NULL(agent->datastate->tasks[index]->var_name, exit(1););
+
   return err_code;
 }
 
@@ -360,8 +454,10 @@ error_code_t
 agent_xml_parse__data(agent_p agent, xml_parser_p xml_parser, int index)
 {
   const char* attribute;
+  const char* attribute2;
   const mxml_node_t *data_node;
   int data_type_size;
+  interpreter_variable_data_t* interp_variable;
   if (xml_parser->node == NULL) {
     return MC_ERR_PARSE;
   }
@@ -374,16 +470,31 @@ agent_xml_parse__data(agent_p agent, xml_parser_p xml_parser, int index)
   }
   data_node = xml_parser->node;
 
-  /* Allocate Return data structure */
-  /* FIXME: This may not be the right place to do this,
-   * but it is safe and leak free. */
-  agent->datastate->tasks[index]->agent_return_data = 
-     agent_return_data_New(); 
+  /* Check to see if this is the return variable */
+  attribute = mxmlElementGetAttr(
+      data_node->parent,
+      "return" );
+  attribute2 = mxmlElementGetAttr(
+      data_node,
+      "name" );
+  if (attribute != NULL && !strcmp(attribute, attribute2)) {
+    /* This variable is the return variable. */
+
+    /* Allocate Return data structure */
+    /* FIXME: This may not be the right place to do this,
+     * but it is safe and leak free. */
+    agent->datastate->tasks[index]->agent_return_data = 
+      interpreter_variable_data_New(); 
+    interp_variable = agent->datastate->tasks[index]->agent_return_data;
+  } else {
+    interp_variable = interpreter_variable_data_New();
+    agent_variable_list_Add(
+        agent->datastate->tasks[index]->agent_variable_list,
+        interp_variable );
+  }
 
 
   /* Attributes we need to parse:
-   * complete       - is the task complete?
-   * server         - The host to perform the task
    * O dim          - dimension of the return data
    * O name         - name of the return variable
    * O persistent   - is the agent persistent for this task?
@@ -395,27 +506,6 @@ agent_xml_parse__data(agent_p agent, xml_parser_p xml_parser, int index)
    *
    * 'O' denotes optional attribute. */
 
-  /* 'complete' */
-  attribute = mxmlElementGetAttr(
-      (mxml_node_t*)data_node,
-      "complete"
-      );
-  CHECK_NULL(attribute, return MC_ERR_PARSE;);
-  agent->datastate->tasks[index]->task_completed = 
-    atoi((char*)attribute);
-  
-  /* 'server' */
-  attribute = mxmlElementGetAttr(
-      (mxml_node_t*)data_node,
-      "server"
-      );
-  CHECK_NULL(attribute, return MC_ERR_PARSE;);
-  agent->datastate->tasks[index]->server_name = 
-    (char*)malloc(sizeof(char) * (strlen(attribute)+1) );
-  strcpy(
-      agent->datastate->tasks[index]->server_name,
-      attribute
-      );
 
   /* 'dim' */
   attribute = mxmlElementGetAttr(
@@ -423,10 +513,10 @@ agent_xml_parse__data(agent_p agent, xml_parser_p xml_parser, int index)
       "dim"
       );
   if (attribute != NULL) {
-    agent->datastate->tasks[index]->agent_return_data->array_dim = 
+    interp_variable->array_dim = 
       atoi((char*)attribute);
   } else {
-    agent->datastate->tasks[index]->agent_return_data->array_dim = 
+    interp_variable->array_dim = 
       0;
   }
 
@@ -436,19 +526,12 @@ agent_xml_parse__data(agent_p agent, xml_parser_p xml_parser, int index)
       "name"
       );
   if (attribute != NULL) {
-    agent->datastate->tasks[index]->var_name = 
+    interp_variable->name = 
       (char*)malloc(sizeof(char)*(strlen(attribute)+1));
     strcpy(
-        agent->datastate->tasks[index]->var_name,
+        interp_variable->name,
         attribute
         );
-  } else {
-    agent->datastate->tasks[index]->var_name = 
-      (char*)malloc(sizeof(char)*(strlen("no-return")+1));
-    strcpy(
-        agent->datastate->tasks[index]->var_name,
-        "no-return"
-        );
   }
 
   /* 'persistent' */
@@ -471,30 +554,30 @@ agent_xml_parse__data(agent_p agent, xml_parser_p xml_parser, int index)
   if (attribute != NULL) {
     CH_STRING_DATATYPE(
         attribute,
-        agent->datastate->tasks[index]->agent_return_data->data_type
+        interp_variable->data_type
         );
     CH_DATATYPE_SIZE(
-        agent->datastate->tasks[index]->agent_return_data->data_type,
+        interp_variable->data_type,
         data_type_size
         );
   } else {
-    agent->datastate->tasks[index]->agent_return_data->data_type = 
+    interp_variable->data_type = 
       CH_UNDEFINETYPE;
     data_type_size = 0;
   }
 
-  if (agent->datastate->tasks[index]->agent_return_data->array_dim == 0) {
+  if (interp_variable->array_dim == 0) {
   /* 'return_value' */
     attribute = mxmlElementGetAttr(
         (mxml_node_t*)data_node,
-        "return_value" );
+        "value" );
     if (attribute != NULL && data_type_size != 0) {
-      agent->datastate->tasks[index]->agent_return_data->return_data =
+      interp_variable->data =
         malloc(data_type_size);
       CH_DATATYPE_STR_TO_VAL(
-          agent->datastate->tasks[index]->agent_return_data->data_type,
+          interp_variable->data_type,
           attribute,
-          agent->datastate->tasks[index]->agent_return_data->return_data
+          interp_variable->data
           );
     }
   } else {
@@ -504,21 +587,7 @@ agent_xml_parse__data(agent_p agent, xml_parser_p xml_parser, int index)
         "ROW",
         1
         );
-    agent_xml_parse__row(agent, xml_parser, index);
-  }
-  /* 'code_id' */
-  attribute = mxmlElementGetAttr(
-      (mxml_node_t*)data_node,
-      "code_id");
-  if (attribute != NULL) {
-    agent->datastate->tasks[index]->code_id = malloc
-      (
-       sizeof(char) * 
-       (strlen(attribute) + 1)
-      );
-    strcpy(agent->datastate->tasks[index]->code_id, attribute);
-  } else {
-    agent->datastate->tasks[index]->code_id = NULL;
+    agent_xml_parse__row(interp_variable, xml_parser, index);
   }
   xml_parser->node = data_node;
   return MC_SUCCESS;
@@ -527,13 +596,12 @@ agent_xml_parse__data(agent_p agent, xml_parser_p xml_parser, int index)
 /* *** */
 /* agent_xml_parse__row */
 error_code_t
-agent_xml_parse__row(agent_p agent, xml_parser_p xml_parser, int index)
+agent_xml_parse__row(interpreter_variable_data_t* interp_variable, xml_parser_p xml_parser, int index)
 {
   int j;
   int data_type_size;
   int tmp;
   int num_elements;
-  agent_task_p task;
   const mxml_node_t* row_node;
 
   if (xml_parser->node == NULL) {
@@ -549,42 +617,41 @@ agent_xml_parse__row(agent_p agent, xml_parser_p xml_parser, int index)
   }
   row_node = xml_parser->node;
 
-  task = agent->datastate->tasks[index];
   /* malloc mem for the task data elements */
   /* First, find the extents of the dimensions */
-  if (task->agent_return_data->array_dim != 0) {
-    task->agent_return_data->array_extent = (int*)
+  if (interp_variable->array_dim != 0) {
+    interp_variable->array_extent = (int*)
       malloc
       (
        sizeof(int) * 
-       task->agent_return_data->array_dim
+       interp_variable->array_dim
       );
     tmp = 0;
     agent_xml_parse__fill_row_data(NULL,
-        task->agent_return_data->data_type,
-        task->agent_return_data->array_extent,
+        interp_variable->data_type,
+        interp_variable->array_extent,
         row_node,
         &tmp);
     num_elements = 1;
-    for (j = 0; j<task->agent_return_data->array_dim; j++) {
-      num_elements *= task->agent_return_data->array_extent[j];
+    for (j = 0; j<interp_variable->array_dim; j++) {
+      num_elements *= interp_variable->array_extent[j];
     }
 
     /* Allocate space for the return data */
     CH_DATATYPE_SIZE
       (
-       task->agent_return_data->data_type,
+       interp_variable->data_type,
        data_type_size
       );
-    task->agent_return_data->return_data =
+    interp_variable->data =
       malloc(num_elements * data_type_size);
 
     /* Get the data */
     tmp = 0;
     agent_xml_parse__fill_row_data(
-        task->agent_return_data->return_data,
-        task->agent_return_data->data_type,
-        task->agent_return_data->array_extent,
+        interp_variable->data,
+        interp_variable->data_type,
+        interp_variable->array_extent,
         row_node,
         &tmp );
   } else { 
@@ -628,6 +695,11 @@ void agent_xml_parse__fill_row_data(
 #endif
     while ( tmp != NULL) {
       switch(type) {
+        case CH_CHARTYPE:
+          if (data != NULL)
+            ((char*)data)[*index] = *(char*)tmp;
+          (*index)++;
+          break;
         case CH_INTTYPE:
           if (data != NULL)
             ((int*)data)[*index] = strtol(tmp, NULL, 0);
@@ -778,7 +850,7 @@ agent_return_xml_parse(agent_p agent)
       "TASK",
       1);
 
-  agent_xml_parse__task(agent, &xml_parser);
+  agent_xml_parse__tasks(agent, &xml_parser);
   return MC_SUCCESS;
 }
 /* message parsing  **********************************************************/
@@ -793,7 +865,7 @@ message_xml_parse(message_p message)
     (
      (mxml_node_t*)xml_parser->root,
      (mxml_node_t*)xml_parser->root,
-     "GAF_MESSAGE",
+     "MOBILEC_MESSAGE",
      NULL,
      NULL,
      MXML_NO_DESCEND
@@ -803,7 +875,7 @@ message_xml_parse(message_p message)
       (
        (mxml_node_t*)xml_parser->root,
        (mxml_node_t*)xml_parser->root,
-       "GAF_MESSAGE",
+       "MOBILEC_MESSAGE",
        NULL,
        NULL,
        MXML_DESCEND
@@ -817,7 +889,7 @@ message_xml_parse(message_p message)
   if(
       strcmp(
         (const char*)xml_get_element_name(xml_parser->node),
-        "GAF_MESSAGE"
+        "MOBILEC_MESSAGE"
         )
     )
   {
